//;(function (window, undefined) {
/**
* View element
*/
var DateSelector = function ( options ) {
	this.options = options;
	if ( isNaN( this.options.firstSelectedOption ) ) {
		this.options.firstSelectedOption = 92; //-- Last 2 months is the default.
	}
	
	this.$element = options.$element;
	this.selectOptions = [];
	this.$customSelectionOption = null;

	this.handleCustomSelectionOption = (function () {
		
		var show = false;
		
		return function ( action ) {
			switch ( action ) {
				case 'show':
					if (show === false) {
						this.$element.insert( this.$customSelectionOption );
						this.$customSelectionOption.selected = true;
						show = true;
					}
					break;

				case 'hide':
					if (show === true) {
						this.$customSelectionOption.remove();
						show = false;
					}
					break;
				default:
					break;
			}
		}
	})();
	
	this.init();
	
};


DateSelector.prototype.init = function () {
	
	this.setValues();
	
};

DateSelector.prototype.destroy = function () {
	
	Element.remove(this.$element);
	
	for (var i in this) {
		delete this[i];
		if (this[i]) {
			this[i] = null;
		}
	}
	
};

DateSelector.prototype.setValues = function () {
	
	this.$customSelectionOption = this.createOption({
		value: 'custom-selection',
		text: this.options.texts.customSelection[this.options.locale]
	});
	
	var valueLastDateSet = false,
	i;
	
	for ( i = 0; i < this.$element.options.length; i++ ) {
		this.selectOptions.push( this.$element.options[i] );

		//-- (this.options.period is the bandwidth of the measurement data. Is null if there are no measurement points)
		
		if ( !this.options.period || i > 0 && (this.options.period[1] - DateSelector.dtm(parseInt( this.$element.options[i].getAttribute('value')) ) < this.options.period[0] && this.options.period[1] - DateSelector.dtm(parseInt( this.$element.options[ i - 1 ].getAttribute('value') )) < this.options.period[0] )) {

			this.$element.options[i].disabled = true;
			
			if ( valueLastDateSet === false && this.options.period ) {
				valueLastDateSet = true;

				this.$element.options[ i - 1 ].setAttribute('value', DateSelector.mtd(parseInt(this.options.period[1] - this.options.period[0])));

			}
			
		}
		
	};

	//-- If the measurement data span a period less than the value in the last option, remove the option all data.
	if ( this.options.period && this.options.period[1] - this.options.period[0] <= DateSelector.dtm(this.selectOptions[this.selectOptions.length - 2].getAttribute('value')) ) {
		this.selectOptions.lastValue().remove();
		this.selectOptions.pop();
	}
	

	if ( valueLastDateSet === false && this.options.period ) {
		
		var lastOption = this.selectOptions.lastValue();
		lastOption.setAttribute('value', DateSelector.mtd(this.options.period[1] - this.options.period[0]));
	}


	//-- Select the initial option.
	this.setInitialSelectedOption();
	
};

DateSelector.prototype.setInitialSelectedOption = function () {
	
	var initialSelectedOption = this.getOptionByValue(this.options.firstSelectedOption),
	i;
	
	if ( !this.selectOptions[initialSelectedOption].disabled ) { //-- if the preferred selected option is not disabled, select the preferred option
		
		this.selectOptions[initialSelectedOption].writeAttribute('selected', 'selected');
	} else { 

		for ( i = initialSelectedOption; i >= 0; i-- ) { //-- loop down through the options from the preferred selected option to 0 and select the first that is not disabled
			if ( !this.selectOptions[i].disabled ) {
				this.selectOptions[i].selected = true;
				return;
			}
		}
		
	}

};

DateSelector.prototype.getOptionByValue = function ( value ) {

	var i;
	
	//-- If the user request to see all data (value == 0), then show the longest period.
	if ( value == 0 ) {
		return this.selectOptions.length - 1;
	}

	//-- Loop through the option and return the index
	for ( i = 0; i < this.selectOptions.length; i ++ ) {
		
		if ( value <= this.selectOptions[i].getAttribute('value') ) {
			return i;
		}

	}
	
	//-- value is greater than the last value of the options, so return the last index.
	return this.selectOptions.length - 1;

};


DateSelector.prototype.createOption = function ( attributes ) {
	
	var $option = new Element('option', {
		'value': attributes.value
	});
	
	Element.update($option, attributes.text);
	
	if ( attributes.disabled ) {
		$option.writeAttribute('disabled');		
	}
	
	if (attributes.cssClass ) {
		$option.writeAttribute('class', attributes.cssClass);
	}

	return $option;
	
};

DateSelector.prototype.getSelectedOption = function () {
	for (i = 0; i < this.selectOptions.length; i++) {
		if ( this.selectOptions[i].selected === true) {
			return this.selectOptions[i];
		};
	}
	
};

/* Converts number of days to milliseconds */
DateSelector.dtm = function ( days ) {
	return ( parseInt( days ) * 24 * 60 * 60 * 1000 );
};

/* Converts milliseconds to number of days  */
DateSelector.mtd = function ( milliseconds ) {
	return ( Math.floor(parseInt( milliseconds ) / 24 / 60 / 60 / 1000) );
};Array.prototype.lastValue = function () {
	
	return this[ this.length - 1 ];
	
};

Array.prototype.max = function(){
	return Math.max.apply( Math, this );
};

Array.prototype.min = function(){
	return Math.min.apply( Math, this );
};


var cl = function ( variable ) {
	
	console && console.log( logText );
	
};

/**
* Inheritance from Child to Parent constructor.
*/
var extendConstructor = function (Child, Parent) {
	var F = function () {};
	F.prototype = Parent.prototype;
	Child.prototype = new F();
	Child.prototype.constructor = Child;
	Child.prototype.uber = Parent.prototype;
};var RangeSlider = function ( options ) {

	
	this.options = options;
	this.slider = null;
	
	this.init();
	
};

RangeSlider.prototype.init = function () {
	
	this.options.container.show();
	
	var that = this;
	
	this.slider = new Control.Slider( this.options.handle, this.options.track, {
		range: $R(0, this.options.range),
		onSlide: function ( value ) {
			that.options.track.fire('rangeSlider:onSlide', {'value': value} ) ;
		},
		onChange: function ( value ) {
			that.options.track.fire('rangeSlider:onChange', {'value': value} ) ;
		}
	});
	
};

RangeSlider.prototype.destroy = function () {
	
	this.slider.dispose();
	for (var i in this) {

		delete this[i];

		if (this[i]) {
			this[i] = null;
		}
	}
	
};

/**
* This method is not used to the moment. Could be used in the future
*/
RangeSlider.prototype.enable = function () {
	this.slider.setEnabled();
	this.options.track.removeClassName('disabled');
	this.options.handle.removeClassName('disabled');

};

RangeSlider.prototype.disable = function () {
	this.slider.setDisabled();
	this.options.track.addClassName('disabled');
	this.options.handle.addClassName('disabled');
};


/**
 * Code for regression extracted from jqplot.trendline.js
 *
 * Version: 1.0.0a_r701
 *
 * Copyright (c) 2009-2011 Chris Leonello
 * jqPlot is currently available for use in all personal or commercial projects
 * under both the MIT (http://www.opensource.org/licenses/mit-license.php) and GPL
 * version 2.0 (http://www.gnu.org/licenses/gpl-2.0.html) licenses. This means that you can
 * choose the license that best suits your project and use it accordingly.
 *
 **/

function regression(x, y, typ) {
	var type = (typ == null) ? 'linear' : typ;
	var N = x.length;
	var slope;
	var intercept;
	var SX = 0;
	var SY = 0;
	var SXX = 0;
	var SXY = 0;
	var SYY = 0;
	var Y = [];
	var X = [];

	if (type == 'linear') {
		X = x;
		Y = y;
	}
	else if (type == 'exp' || type == 'exponential') {
		for (var i = 0; i < y.length; i++) {
			// ignore points <= 0, log undefined.
			if (y[i] <= 0) {
				N--;
			}
			else {
				X.push(x[i]);
				Y.push(Math.log(y[i]));
			}
		}
	}

	for (var i = 0; i < N; i++) {
		SX = SX + X[i];
		SY = SY + Y[i];
		SXY = SXY + X[i] * Y[i];
		SXX = SXX + X[i] * X[i];
		SYY = SYY + Y[i] * Y[i];
	}

	slope = (N * SXY - SX * SY) / (N * SXX - SX * SX);
	intercept = (SY - slope * SX) / N;

	return [slope, intercept];
}

function linearRegression(X, Y) {
	var ret;
	ret = regression(X, Y, 'linear');
	return [ret[0], ret[1]];
}

function expRegression(X, Y) {
	var ret;
	var x = X;
	var y = Y;
	ret = regression(x, y, 'exp');
	var base = Math.exp(ret[0]);
	var coeff = Math.exp(ret[1]);
	return [base, coeff];
}

function fitData(data, typ) {
	var type = (typ == null) ? 'linear' : typ;
	var ret;
	var res;
	var x = [];
	var y = [];
	var ypred = [];

	for (i = 0; i < data.length; i++) {
		if (data[i] != null && data[i][0] != null && data[i][1] != null) {
			x.push(data[i][0]);
			y.push(data[i][1]);
		}
	}

	if (type == 'linear') {
		ret = linearRegression(x, y);
		for (var i = 0; i < x.length; i++) {
			res = ret[0] * x[i] + ret[1];
			ypred.push([x[i], res]);
		}
	}
	else if (type == 'exp' || type == 'exponential') {
		ret = expRegression(x, y);
		for (var i = 0; i < x.length; i++) {
			res = ret[1] * Math.pow(ret[0], x[i]);
			ypred.push([x[i], res]);
		}
	}
	return {
		data: ypred,
		slope: ret[0],
		intercept: ret[1]
	};
};



;var Chart = function () {
	
	this.init();
	
};


Chart.prototype.init = function () {

	this.locale = this.options.locale;
	this.graphic = null;
	this.scrollSpeed = 14 * 24 * 60 * 60 * 1000, //-- 7 days, TODO: this wil be a variable.
	this.bandWidthX = {};
	this.maxZoom = 7 * 24 * 60 * 60 * 1000;
	this.$container = $(this.options.container);
	this.dataTouchZoomStart = {};
	this.dataDragStart = {};
	this.dataOnActionStart = {};
	this.dragging = false;
	this.overflow = null; //-- Boolean, true if the bandwidth of measurement dates is larger than the minimum bandwidth of dates. Used for disabling events.
	this.tooltip = this.options.tooltip === false ? false : true; //-- Boolean to show the tooltip, default is true.
	
	//-- Translations for the locales
	var lang = {};
	
	switch ( this.locale ) { //-- created a switch so it is easier to extend for other languages
		case 'nl':
			lang.months = ['januari', 'febuari', 'maart', 'april', 'mei', 'juni', 'juli', 'augustus', 'september', 'oktober', 'november', 'december'];
			lang.shortMonths = ['jan', 'feb', 'maa', 'apr', 'mei', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'dec'];
			lang.weekdays = ['zondag', 'maandag', 'dinsdag', 'woensdag', 'donderdag', 'vrijdag', 'zaterdag'];
			break;
		default: //-- English is the default and the default of Highcharts, so do nothing
			break;
	}

	if ( lang.months || lang.shortMonths || lang.weekdays ) {
		Highcharts.setOptions({
			lang: lang
		});
	}
	
};

Chart.prototype.draw = function ( plotOptions, yAxisLabelsEnabled ) {
	
	var that = this;

	this.graphic = new Highcharts.Chart({
		chart: {
			renderTo: this.options.container,
			plotBackgroundColor: '#f5f4fa',
			plotBorderColor: '#d1d1d1',
			plotBorderWidth: 2,
			spacingBottom: this.options.legend === true ? 50 : 10,
			spacingRight: this.options.legend === true ? 15 : 0,
			marginLeft: 50,
			width: Prototype.Browser.IE ? this.options.container.getWidth() : null,
			height: Prototype.Browser.IE ? this.options.container.getHeight() : null
		},
		ignoreHiddenSeries: true,
		credits: {
			enabled: false
		},
		legend: {
			enabled: this.options.legend,
			borderWidth: 0,
			itemStyle: {
				'textDecoration': 'underline',
				'color': '#22aeeb'
			},
			align: 'left',
			floating: true,
			x: 30,
			y: 30,
			itemHiddenStyle: {
				'color': '#666'
			},
			itemHoverStyle: {
				'color': '#000',
				'cursor': 'pointer !important'
			},
			style: {
				'cursor': 'pointer'
			},
			symbolPadding: 3
		},
		colors: ['#00a2e3', '#f00'],
		title: {
			text: null
		},
		xAxis: {
			type: 'datetime',
			minRange: 7 * 24 * 60 * 60 * 1000, //-- 1 week.
			gridLineWidth: 1,
			labels: {
				rotation: -45,
				y: 15,
				align: 'right',
				style: {
					fontSize: '13px'
				}
			},
			minTickInterval: 1 * 24 * 60 * 60 * 1000, //-- 1 day
			tickPixelInterval: 40,
			id: 'xAxis',
			dateTimeLabelFormats: {
				day: '%e %b',
				week: '%e %b'
			}
		},
		yAxis: {
			title: {
				text: null
			},
			minorGridLineWidth: 2,
			id: 'yAxis',
			labels: {
				formatter: function () {
					return this.value + ' ' + that.options.units;
				},
				enabled: yAxisLabelsEnabled
			},
			startOnTick: false,
			endOnTick: false,
			minRange: 2
		},
		plotOptions: plotOptions,
		tooltip: {
			enabled: this.tooltip,
			formatter: function () {
				var htmlString = '';
				htmlString += '<span style="line-height: 1.5em;">' + Highcharts.dateFormat('%A %e %b %Y', this.x, true) + '</span>';
				htmlString += '<br />';
				htmlString += '<span style="line-height: 1.5em; color: ' + this.series.color + ';">' + this.series.name + ': <b>' + this.y + ' ' + that.options.units + '</b></span>';
				
				return htmlString;
			}
		}
	});
	
};

Chart.prototype.getZoomStartDate = function ( startDate ) {
	
	startDate = parseInt( startDate );
	
	//-- Get the start date
	if ( isNaN( startDate ) ) {//-- The user requests to see all data

		return this.data[0][0]; //-- start date is the first point of all points
		
	} else { //-- the user has requested to see a period
		
		return startDate;
		
	}
	
};

/**
* Stores the necessary data on the touch zoom start event.
*/
Chart.prototype.setDataTouchZoomStart = function () {
	//cl('setDataTouchZoomStart');
	
	this.dataTouchZoomStart.origZoom = this.graphic.xAxis[0].getExtremes(),
	this.dataTouchZoomStart.origBandWidth = this.dataTouchZoomStart.origZoom.max - this.dataTouchZoomStart.origZoom.min,
	this.dataTouchZoomStart.origMiddle = this.dataTouchZoomStart.origZoom.min + this.dataTouchZoomStart.origBandWidth / 2;
};

Chart.prototype.touchZoom = function ( transformEvent ) {
	//cl('touchZoom')
	
	var newZoom,
	zoomScale,
	tooMuch;
	
	zoomScale = 1 + ( 1 - transformEvent.scale);
	
	newZoom = {};

	newZoom.min = this.dataTouchZoomStart.origMiddle - (this.dataTouchZoomStart.origBandWidth / 2) * zoomScale;
	newZoom.max = this.dataTouchZoomStart.origMiddle + (this.dataTouchZoomStart.origBandWidth / 2) * zoomScale;

	//-- Check the bounderies.
	
	if (newZoom.min < this.bandWidthX.start) { //-- zoom out has reached left boundery
		tooMuch = this.bandWidthX.start - newZoom.min;
		newZoom.min = this.bandWidthX.start;
		
		if ( newZoom.max < this.bandWidthX.end ) {
			newZoom.max += tooMuch;
		}
	}

	if ( newZoom.max > this.bandWidthX.end ) { //-- zoom out has reached right boundery
		tooMuch = newZoom.max - this.bandWidthX.end;
		newZoom.max = this.bandWidthX.end;

		if ( newZoom.min > this.bandWidthX.start ) {
			newZoom.min -= tooMuch;
		}

	}
	
	if ( newZoom.max - newZoom.min < this.maxZoom ) { //-- don't zoom in if the graphic has reached its maximum zoom level, as declared in chart.maxZoom
		return;
	}
	
	var extremes = this.graphic.xAxis[0].getExtremes();
	
	if ( Math.abs(newZoom.min - this.graphic.xAxis[0].getExtremes().min) > 1 * 24 * 60 * 60 * 1000 ) {
		this.zoom( newZoom.min, newZoom.max );
	}
	
	
};

Chart.prototype.touchZoomEnd = function () {
	//cl('touchZoomEnd');
	
	this.zoomPhase = 1;
};

Chart.prototype.setDataDragStart = function ( dragStartEvent ) {
	//cl('setDataDragStart');
	
	this.dataDragStart.positionX = dragStartEvent.position.x;
	
	/*
	if (this.graphic.xAxis[0].getExtremes().min > this.data.lastValue()[0]) {}
	*/
	
};

Chart.prototype.drag = function ( dragEvent ) {
	//cl('drag');
	
	var amount; 
	
	amount = dragEvent.position.x - this.dataDragStart.positionX;
	
	if ( this.scroll( amount ) ) {
		if ( this.dragging === false ) {
			this.$container.fire( 'chart:onDrag' );
			this.dragging = true;
		}
		
	}
	
	this.dataDragStart.positionX = dragEvent.position.x;

};

Chart.prototype.dragEnd = function () {
	//cl('dragEnd');
	
	this.dragging = false;
};

/**
* Zooming by the range slider
*/
Chart.prototype.guiZoom = function ( event, rangeSliderRange ) {
	//cl('guiZoom');
	
	//--TODO: Create this: stop the zoom if the last point is not visible
	
	var minZoom,
	bandWidthNew,
	origZoom,
	origBandWidth,
	origMiddle,
	newZoom,
	tooMuch;
	

	minZoom = this.bandWidthX.end - this.bandWidthX.start; //-- value when the graphic is zoomed out to the max.
	
	bandWidthNew = minZoom - ( ( event.memo.value / rangeSliderRange ) * ( minZoom - this.maxZoom ) ); //-- the new bandWidth to be displayed
	
	origZoom = this.graphic.xAxis[0].getExtremes(); //-- get the current band width data
	
	origBandWidth = origZoom.max - origZoom.min; //-- calculate the current band width 
	
	origMiddle = origZoom.min + origBandWidth / 2; //-- the middle of the current bandWidth
	
	newZoom = {
		start: origMiddle - bandWidthNew / 2,
		end: origMiddle + bandWidthNew / 2
	}
	
	if ( newZoom.start < this.bandWidthX.start ) {
		tooMuch = this.bandWidthX.start - newZoom.start;
		newZoom.start = this.bandWidthX.start;
		newZoom.end += tooMuch;
	}

	/*
	if ( newZoom.end > this.bandWidthX.end ) {
		tooMuch = newZoom.end - this.bandWidthX.end;
		newZoom.end = this.bandWidthX.end;
		newZoom.start -= tooMuch;
	}
	*/
	
	this.zoom(newZoom.start, newZoom.end);
	
};

Chart.prototype.scroll = function ( netScrollAmount /* in pixels */) {
	
	var xExtremes = this.graphic.xAxis[0].getExtremes();
	var xBandWidth = xExtremes.max - xExtremes.min;
	
	var ratio = xBandWidth / this.graphic.xAxis[0].width;
	
	
	scrollAmount = -(netScrollAmount * ratio);
	
	if ( scrollAmount < 0 ) {
		
		if ( xExtremes.min === this.options.firstXValue ) {
			
			return false;
		}
		
		if ( (xExtremes.min + scrollAmount) <= this.options.firstXValue ) {
			this.graphic.xAxis[0].setExtremes( this.options.firstXValue, this.options.firstXValue + xBandWidth, true, false );
			return true;
		}
		
		this.graphic.xAxis[0].setExtremes( xExtremes.min + scrollAmount, xExtremes.max + scrollAmount, true, false );
		return true;
	}

	if ( scrollAmount > 0 ) {
		if (!this.lastDateX && ( xExtremes.min === this.options.lastXValue - ( xBandWidth * .05 ) )) {
			//cl(0);
			return false;
		}

		if (!this.lastDateX && ( xExtremes.min + scrollAmount > this.options.lastXValue - ( xBandWidth * .05 ) )) {
			//cl(1);
			this.graphic.xAxis[0].setExtremes( xExtremes.min, xExtremes.min + xBandWidth, true, false );
			return true;
		}


		if ( this.lastDateX && ( xExtremes.min === this.lastDateX - ( xBandWidth * .05 ) ) ) {
			//cl(2);
			return false;
		}

		if ( this.lastDateX && ( xExtremes.min + scrollAmount >= this.lastDateX - ( xBandWidth * .05 ) ) ) {
			//cl(3);
			this.graphic.xAxis[0].setExtremes( this.lastDateX - ( xBandWidth * .05 ), this.lastDateX - ( xBandWidth * .05 ) + xBandWidth, true, false );
			return true;
		}

		
		this.graphic.xAxis[0].setExtremes( xExtremes.min + scrollAmount, xExtremes.max + scrollAmount, true, false );

		return true;
	
	}
	
};

Chart.prototype.zoom = function (startDate, endDate) { //-- TODO: make this an object.
	//cl('zoom');
	
	this.graphic.xAxis[0].setExtremes( startDate, endDate, true, false );

};

/**
* Calculates the zoom level for a given zoom scale
* Used to set the value of the range slider.
*/
Chart.prototype.getZoomLevel = function ( zoomScale ) {
	
	var zoom,
	zoomBandWidth,
	chartBandWidth,
	value;

	zoom = this.graphic.xAxis[0].getExtremes(); //-- get data of current view.

	zoomBandWidth = zoom.max - zoom.min; // the bandwidth of the current zoom level
	
	chartBandWidth = this.bandWidthX.end - this.bandWidthX.start; //-- the total bandwidth of the chart.
	
	value = zoomScale - ( ( zoomBandWidth / chartBandWidth ) * zoomScale ); //-- the calculation of the value.
	

	return parseInt( value );

};

Chart.prototype.setDataOnActionStart = function () {};

/**
* If there are no measurement points, this function creates a grid by setting the extremes of the x-axis from one month ago to today.
*/
Chart.prototype.setGridNoMeasurements = function () {
	this.graphic.xAxis[0].setExtremes( Date.parse ( new Date() ) - 30 * 24 * 60 * 60 * 1000, Date.parse ( new Date() ), true, true );
	this.graphic.yAxis[0].setExtremes( 0, 200, true, true );
};var SingleMeasurementChart = function ( options ) {
	this.options = options;

	this.dataOnActionStart = {};
	
	this.init();
	
};

extendConstructor(SingleMeasurementChart, Chart);

SingleMeasurementChart.prototype.init = function () {
	//cl('init');
	
	this.legendTexts = {
		target: {
			en: 'Goal',
			nl: 'Doel'
		},
		trend: {
			en: 'Trend',
			nl: 'Trend'
		}
	};
	
	this.uber.init.apply( this ); //-- Call the init of the parent in the context of this class.
	
	this.data = $A( this.options.data );
	
	if ( this.data.length > 0 ) {
		this.bandWidthX.start = this.data[0][0];
	}
	
	
	this.goal = this.options.goal;
	
	//-- set the variables 
	if ( this.options.settings.showTrend === true ) {
		this.dataOnActionStart.trendLineVisible = true;
	} else {
		this.dataOnActionStart.trendLineVisible = false;
	}

	if ( this.options.settings.showValue === true ) {
		this.dataOnActionStart.measurementLineVisible = true;
	} else {
		this.dataOnActionStart.measurementLineVisible = false;
	}

	this.draw();

};

SingleMeasurementChart.prototype.draw = function () {
	//cl('draw');
	var that = this,
	measurementLine,
	goalLine,
	yAxisLabelsEnabled;
	
	if ( !this.goal && $A(this.data).length === 0 ) {
		yAxisLabelsEnabled = false;
	} else {
		yAxisLabelsEnabled = true;
	}
	
	var plotOptions = {
		line: {
			events: {
				legendItemClick: function ( e ) {
					
					var finishPoint,
					targetLine = that.graphic.get( 'target' );
					
					if ( this.visible === false && this.name.toLowerCase() === that.legendTexts.trend[that.options.locale].toLowerCase() ) { //-- click on the legend activates the trend line
						this.setVisible(true, false);
						that.dataOnActionStart.trendLineVisible = true;
						that.$container.fire( 'chart:legendShowTrendClick' );
						return false;
					} else {
						that.dataOnActionStart.trendLineVisible = false;
					}
					
					//-- Click on the targetLine
					if ( this.name.toLowerCase() === that.legendTexts.target[that.options.locale].toLowerCase() ) {
						this.setVisible();
						var trendLine = that.graphic.get('trend');
						if ( trendLine && trendLine.visible === true ) {
							that.dataOnActionStart.trendLineVisible = true;
						} else {
							that.dataOnActionStart.trendLineVisible = false;
						}
						
						trendLine && trendLine.setVisible(false, true);
						that.manageFinishState();
						that.$container.fire( 'chart:legendTargetClick' );

						return false;
					}
					
					this.setVisible();
					that.manageFinishState( ); 
					return false;
					
				}
			},
			stickyTracking: false,
			marker: {
				states: {
					hover: {
						enabled: this.tooltip
					}
				}
			}
		}
	};
	
	this.uber.draw.apply( this, [plotOptions, yAxisLabelsEnabled] ); //-- Call the init of the parent in the context of this class.
	
	measurementLine = this.graphic.addSeries({
		name: this.options.legendText,
		id: 'measurementLine',
		type: 'line',
		data: $A(this.data),
    enableMouseTracking: false, // EV: added this to test mobile performance
    shadow: false, // EV: added this to test mobile performance
    marker: {enabled: true},
		index: 4,
		dataGrouping: {
			enabled: false
		},
		showInLegend: this.options.legend
	});
	
	if ( this.options.settings.showValue === false ) {
		measurementLine.hide();
	}
	
	if ( this.goal ) {

		var points = [];
		
		if ( $A(this.data).length === 0 ) {

			var today = new Date();
			points.push({
				x: Date.parse(today),
				y: this.goal
			}, {
				x: Date.parse(today) + 30 * 24 * 60 * 60 * 1000,
				y: this.goal
			});
			
			
		} else if ( $A(this.data).length === 1 ) {

			points.push({
				x: this.data[0][0] - ( 1 * 24 * 60 * 60 * 1000 ),
				y: this.goal
			}, {
				x: this.data[0][0] + ( 30 * 24 * 60 * 60 * 1000 ),
				y: this.goal
			});
			
		} else {

			points.push({
				x: this.data[0][0],
				y: this.goal
			}, {
				x: this.data.lastValue()[0] + ( ( this.data.lastValue()[0] - this.data[0][0] ) / 2 ),
				y: this.goal
			});
			
		}
		
		goalLine = this.graphic.addSeries({
      shadow: false, // EV: added this to test mobile performance
			name: this.legendTexts.target[this.options.locale],
			id: 'target',
			type: 'line',
			data: [
				{
					x: points[0]['x'],
					y: points[0]['y'],
					marker: {
						enabled: false
					},
					id: 'first-point-target'
				},
				{
					x: points[1]['x'],
					y: points[1]['y'],
					marker: {
						enabled: false
					},
					id: 'last-point-target'
				}
			],
			index: 3,
			legendIndex: 1,
			enableMouseTracking: false,
			showInLegend: this.options.legend
		});
	}
	
	if ( this.options.settings.showGoal === false ) {
		goalLine.hide();
	}

	if ( !this.goal && $A(this.data).length === 0 ) {
		this.setGridNoMeasurements();
	}
	
	this.maybeRedrawTargetLine();

	
};

SingleMeasurementChart.prototype.drag = function ( dragEvent ) {

	//-- Call the parent to perform the drag...
	this.uber.drag.call( this, dragEvent );
	
	//-- ...and check if the target line needs redrawing.
	this.maybeRedrawTargetLine();
	
};

/**
* Shows or hides the finish flag
*/
SingleMeasurementChart.prototype.manageFinishState = function () {
	//cl('manageFinishState');
	
	var finishPoint = this.graphic.get( 'finishPoint' ),
	trendLine = this.graphic.get( 'trend' ),
	targetLine = this.graphic.get( 'target' );

	if ( !finishPoint || !trendLine || !targetLine ) {
		return;
	}

		
	if ( trendLine.visible && targetLine.visible ) {
		finishPoint.show();
	} else {
		finishPoint.hide();
	}
		
};

SingleMeasurementChart.prototype.maybeRedrawTargetLine = function () {
	//console.log('maybeRedrawTargetLine');
	
	if (!this.goal) {
		return;
	}

	var extremes = this.graphic.xAxis[0].getExtremes(),
	firstPointTarget = this.graphic.get('first-point-target'),
	lastPointTarget = this.graphic.get('last-point-target');
	
	if ( extremes.min < firstPointTarget.x ) {
		this.fixExtremesY();
		firstPointTarget.update( [extremes.min, firstPointTarget.y], true, false );
		this.releaseExtremesY();
	}

	if ( extremes.max > lastPointTarget.x ) {
		this.fixExtremesY();
		lastPointTarget.update( [extremes.max, lastPointTarget.y], true, false );
		this.releaseExtremesY();
	}
	
}

/**
* Calculates the end date of the initial zoom.
*/
SingleMeasurementChart.prototype.getZoom = function ( startDate, xCross ) {
	//cl('getZoom');
	var zoom = {},
	extra;
	
	zoom.startDate = this.getZoomStartDate( startDate );
	
	//-- Get the end date
	if ( xCross && this.dataOnActionStart.trendLineVisible === true ) { //-- if trend and target cross, that will be the end date of the zoom
		
		extra = (xCross - zoom.startDate) * 24 / this.graphic.xAxis[0].width;
		
		zoom.endDate = xCross + extra;

	} else { //-- show 50% extra of the selected period 
		
		lastXPoint = this.data.lastValue()[0];

		extra = ( lastXPoint - zoom.startDate ) / 2; //-- 50%, TODO: variabelize
		
		zoom.endDate = lastXPoint + extra;
		
	}
	
	this.lastDateX = zoom.endDate; //-- no right scrolling after this point
	
	return zoom;
	
};

SingleMeasurementChart.prototype.setMaxScrollingDate = function ( lastCrossing ) {
		
	this.lastDateX = lastCrossing;
	
	//-- TODO: what if there is no lastCrossing?
	
};


SingleMeasurementChart.prototype.setDataOnActionStart = function () {

	var trendLine = this.graphic.get('trend'),
	measurementLine = this.graphic.get('measurementLine');
	
	if ( trendLine && trendLine.visible || this.dataOnActionStart.trendLineVisible === true ) {
		this.dataOnActionStart.trendLineVisible = true;
	} else {
		this.dataOnActionStart.trendLineVisible = false;
	};
	
	if ( measurementLine && measurementLine.visible ) {
		this.dataOnActionStart.measurementLineVisible = true;
	} else {
		this.dataOnActionStart.measurementLineVisible = false;
	}
	
	
	if ( trendLine && measurementLine && measurementLine.visible ) {
		trendLine.setVisible( false, true );
		this.manageFinishState();
	}
	
	
};

SingleMeasurementChart.prototype.drawTrendLine = function ( trendLineData ) {
	//cl('drawTrendLine');

	//-- Cache the extremes of the x and y axes.
	var extremesX = this.graphic.xAxis[0].getExtremes(),
	extremesY = this.graphic.yAxis[0].getExtremes();
	
	if ( this.dataOnActionStart.measurementLineVisible === true ) {
		this.graphic.yAxis[0].setExtremes(extremesY.min, extremesY.max); //-- don't let the drawing of the trend line influence the width of the graphic.
	}

	var trendLine = this.graphic.get( 'trend' ),
	targetLine = this.graphic.get( 'target' ),
	oldFinishPoint;

	//-- remove the old trend line, if there is one
	if ( trendLine ) {
		trendLine.remove(false);
	}
	
	var trendLinePoints = []; //-- two points defining the trend line

	var left = trendLineData.slope * extremesX.min + trendLineData.intercept; //-- y value of first point
	var right = trendLineData.slope * extremesX.max + trendLineData.intercept; //-- y value of second point

	trendLinePoints.push( [extremesX.min, left] );
	
	//-- If the trend line and the goal line cross and the goal line is visible, than the right point is that crossing
	if ( trendLineData.xCross && targetLine.visible === true ) {
		trendLinePoints.push( [trendLineData.xCross, this.goal] );
	} else {
		trendLinePoints.push( [extremesX.max, right] );
	}
	
	//-- create the new trend line
	trendLine = this.graphic.addSeries({
    shadow: false, // EV: added this to test mobile performance
		name: this.legendTexts.trend[this.options.locale],
		id: 'trend',
		data: $A(trendLinePoints),
		legendIndex: 2,
		marker: {
			enabled: false,
			states: {
				hover: {
					enabled: false
				}
			}
		},
		color: '#fece0f',
		enableMouseTracking: false,
		showInLegend: this.options.legend
		
	});
	
	//-- show the new trend line
	
	if ( this.dataOnActionStart.trendLineVisible === true ) {
		trendLine.setVisible(true, false);
	} else {
		trendLine.setVisible(false, false);
	}
	
	

	if ( trendLineData.xCross ) { //-- only if the target line and the trend line cross, set a new finish point
		oldFinishPoint = this.graphic.get('finishPoint');
		if ( oldFinishPoint ) {
			oldFinishPoint.remove();
		}
		
		this.graphic.addSeries({
			id: 'finishPoint',
			data: [[trendLineData.xCross, this.goal]],
			animation: false,
			marker: {
				symbol: this.options.finishSymbol
			},
			showInLegend: false,
			name: this.legendTexts.trend[this.options.locale],
			color: '#00a2e3'
		}, false, false).hide();
		
		this.manageFinishState(); //-- show or hide the finish flag
		
		
		//-- check if we need to update the targetLine. (Only if the crossing is right from the max x value)
		var targetLine = this.graphic.get('target');
		if ( targetLine.xData.lastValue() < trendLineData.xCross ) {
			targetLine.addPoint({
				x: trendLineData.xCross,
				y: this.goal,
				marker: {
					enabled: false
				}
			}, false, false, false);
		}
		

	} else { //-- the target and the trendline do not cross in the future, remove the old finish point
	
		oldFinishPoint = this.graphic.get('finishPoint');
		if ( oldFinishPoint ) {
			oldFinishPoint.remove();
		}

	}
	
	this.releaseExtremesY();
	
	return trendLine;
	
};


//-- TODO: this can be moved to the parent: Chart
SingleMeasurementChart.prototype.releaseExtremesY = function () {
	//cl('releaseExtremesY');
	
	this.graphic.yAxis[0].setExtremes(null, null, false, false);
};

//-- TODO: this can be moved to the parent: Chart
SingleMeasurementChart.prototype.fixExtremesY = function () {
	var extremesY = this.graphic.yAxis[0].getExtremes();
	
	this.graphic.yAxis[0].setExtremes(extremesY.min, extremesY.max);

};

/**
* Only called on a change of the date range selector, or to get the first view after page load, so that one of the standard settings are shown.
* If necessary redraws all lines: value, target and trend.
*/
SingleMeasurementChart.prototype.doInitZoom = function ( trendLineData, startDate ) {

	var zoom = this.getZoom( startDate, trendLineData ? trendLineData.xCross : null ); //-- calculates the end date of the zoom
	
	this.bandWidthX.end = zoom.endDate; //-- set the boundary in a class variable.

	this.setDataOnActionStart();
	this.zoom( zoom.startDate, zoom.endDate ); //-- do the actual zoom (this will update the view)
	
	if ( trendLineData && ( !isNaN( trendLineData.slope ) || !isNaN( trendLineData.intercept ) ) ) { //-- Don't draw the trendline if there is no valid data (for instance when there is only one measurement point )
		this.drawTrendLine( trendLineData ); //-- draw the new trendline.
	}

	/**
	* Set the overflow. True if bandwidth of measurement dates is larger than minimum range to be displayed
	* TODO: This is not correct if goal and trend cross.
	*/
	if ( ( ( this.options.lastXValue - this.options.firstXValue ) < ( this.graphic.xAxis[0].minRange ) && trendLineData.xCross === null ) || this.data.length < 2 ) {
		this.overflow = false;
	} else {
		this.overflow = true;
	}
	
	this.maybeRedrawTargetLine();
	
};var CompetitiveGraphChart = function ( options ) {
	this.options = options;
	
	this.dataOnActionStart = {};
	
	this.init();
	
};

extendConstructor( CompetitiveGraphChart, Chart );

CompetitiveGraphChart.prototype.init = function () {
	
	this.legendTexts = {
		group: {
			en: this.options.legendGroupText,
			nl: this.options.legendGroupText
		},
		user: {
			en: this.options.legendText,
			nl: this.options.legendText
		}
	};
	
	this.uber.init.apply( this ); //-- Call the init of the parent in the context of this class.


	this.data = this.options.data;


	if ( this.options.period !== null ) {
		this.bandWidthX = {
			start: this.options.period[0],
			end: this.options.period[1]
		};
	}
	
	this.draw();
	
};

CompetitiveGraphChart.prototype.draw = function () {
	//cl('draw');
	
	var that = this,
	measurementLine,
	plotOptions = {
		line: {
			stickyTracking: false,
			marker: {
				states: {
					hover: {
						enabled: this.tooltip
					}
				}
			}
		}
	};
	
	if ( this.options.period === null ) {
		yAxisLabelsEnabled = false;
	} else {
		yAxisLabelsEnabled = true;
	}

	this.uber.draw.apply( this, [plotOptions, yAxisLabelsEnabled] ); //-- Call the init of the parent in the context of this class.
	
	measurementLine = this.graphic.addSeries({
		name: this.legendTexts.user[this.locale],
		id: 'measurementLine',
		type: 'line',
		data: $A(this.data.measurements),
		index: 4,
		dataGrouping: {
			enabled: false
		},
		showInLegend: this.options.legend
	});
	
	groupLine = this.graphic.addSeries({
		name: this.legendTexts.group[this.locale],
		id: 'groupLine',
		type: 'line',
		data: $A(this.data.groupAverages),
		index: 4,
		dataGrouping: {
			enabled: false
		},
		color: '#0e5406',
		showInLegend: this.options.legend
	});
	
	if ( this.options.period === null ) {
		this.setGridNoMeasurements();
		this.overflow = false;
	}
	
};

CompetitiveGraphChart.prototype.setMaxScrollingDate = function () {
	

};


/**
* Only called on a change of the date range selector, or to get the first view after page load, so that one of the standard settings are shown.
*/
CompetitiveGraphChart.prototype.doInitZoom = function ( startDate, endDate ) {

	this.bandWidthX.end = endDate; //-- set the boundary in a class variable.

	this.setDataOnActionStart();
	this.zoom( startDate, endDate ); //-- do the actual zoom (this will update the view)
	
	/**
	* True if bandwidth of measurement dates is larger than minimum range to be displayed
	*/
	if ( this.options.lastXValue - this.options.firstXValue < this.graphic.xAxis[0].minRange ) {
		this.overflow = false;
	} else {
		this.overflow = true;
	}
	

	
	
};var Model = function  ( options ) {
	this.name = 'Model';
	
};

Model.prototype.init = function () {
	
	this.firstXValue = null;
	this.lastXValue = null;
	
	this.parseJSON();
	
};

/**
* Must be overriden in child class.
*/
Model.prototype.parseJSON = function () {};


/**
* Please note, this function is not tied to an object instantiation.
*/
Model.getTrendLineData = function ( data ) {
	return fitData( data );
};/**
* Extends Model
*/
;var SingleMeasurementModel = function ( options ) {
	
	this.name = 'SingleMeasurementModel';
	
	this.options = options;
	
	this.lastCrossing = null;
	
	this.data = []; //-- parsed data in Highcharts readable format.
	this.goal = null;
	
	this.locale = null;
	
	this.totalNumberOfMeasurements = null;
	
	this.init();
	
	
	
};

extendConstructor(SingleMeasurementModel, Model);


SingleMeasurementModel.prototype.init = function () {
	
	this.uber.init.apply( this ); //-- Call the init of the parent in the context of this class.
	
	this.totalNumberOfMeasurements = this.data.length;


	
	//-- Set the necessary variables
	this.goal = null || this.options.data.goal;
	this.locale = this.options.data.locale;
	
	if ( $A(this.data).length > 0 ) {
		this.firstXValue = this.data[0][0];
		this.lastXValue = this.data.lastValue()[0]; //-- cache the last date measured.
	}

	this.setLastCrossing();
	
};

SingleMeasurementModel.prototype.parseJSON = function () {
	var i, //-- Counter
	measurement,
	dateArray;
	
	//-- Loop trough the measurements and parse them in the for Highcharts readable format.
	for (i = 0; i < this.options.data.measurements.length; i++) {
		
		measurement = []; //-- temporary helper array.
		dateArray = this.options.data.measurements[i].created_on.split('/');
		
		measurement.push( Date.UTC( dateArray[0], ( dateArray[1] - 1 ), dateArray[2]) ); //-- Convert the date to a Unix date in milliseconds.
		measurement.push( this.options.data.measurements[i].value );
		
		this.data.push( measurement );
		
	}
	
};

/**
* Calculates the last crossing of the trendline of two last measurement points and the target line (if any)
*/
SingleMeasurementModel.prototype.setLastCrossing = function () {
	
	var data,
	trendLineData,
	lastCrossing;

	//-- Set the maxScrolling Date
	if ( this.goal && this.data.length > 2 ) {
		data = this.data.slice( this.data.length - 2, this.data.length ); //-- Get the last two points...
		trendLineData = Model.getTrendLineData( data );
	
		lastCrossing = this.getXCross( trendLineData ); //-- ...and calculate the last possible crossing.
		if ( lastCrossing ) { 
			this.lastCrossing = parseInt( lastCrossing ); //-- TODO: Maybe cut this to 0.00 hours
		} //-- this.lastCrossing remains null
	} //-- this.lastCrossing remains null
	
};

/**
* Calculates if a line crosses the target line
*/
SingleMeasurementModel.prototype.getXCross = function ( lineData ) {

	var xCross;
	
	if ( !this.goal ) {
		return null;
	}
	
	xCross = Math.round( ( this.goal - lineData.intercept ) / lineData.slope ) /* x value of intersect trendline and target */
	if ( xCross > Math.round( this.lastXValue ) ) { //-- the crossing should be in the future
		return xCross;
	} else {
		return null;
	}
	
};

/**
* Gets the necessary data to draw the trend line and calculates if and where the trend line crosses the target line.
* zoom.min & zoom.max
*/
SingleMeasurementModel.prototype.getTrendLineData = function ( zoom ) {

	console.log(zoom);

	var trendData,
	trendLineData,
	indexStart = null,
	indexStop = null,
	i,
	xCross;

	/* This logic will be done in the calling function
	//-- The zoom range for the calculation of the trend line.
	if ( startDate ) {
		zoom = {
			min: startDate,
			max: null
		}
	} else if ( startDate !== undefined ) { //-- startDate isNaN

		zoom = {
			min: this.data[0][0],
			max: null
		}
		
	} else {
		zoom = this.chart.graphic.xAxis[0].getExtremes();
	}
	
	*/

	for ( i = 0; i < this.data.length; i ++ ) {

		if ( parseInt( this.data[i][0] ) >= parseInt( zoom.min ) && typeof indexStart !== 'number') {

			indexStart = i; //-- use only the data with this index as starting point for calculating the trend line

			if ( zoom.max === null ) { //-- if there is no max zoom set, then the last measurement will be taken
				break;
			}

		}

		if ( this.data[i][0] > parseInt( zoom.max ) && typeof indexStop !== 'number' ) {
			indexStop = i;
			break;
		}
		
	}
	
	
	
	if ( !indexStop ) {
		indexStop = this.data.length
	}
	
	if ( indexStop - indexStart === 1 ) { //-- trendline will be based on one point, which is not possible so we have take in extra points for calculating the trendline.

		if ( indexStop + 1 === this.data.length ) {
			indexStart = indexStart - 2;
		} else if ( indexStart - 1 === -1 ) {
			indexStop = indexStop + 2;
		} else {
			indexStart --;
			indexStop ++;
		}
		
	} else if ( indexStop === indexStart ) { //-- there are no data points in the view, so the trendline will be the same as the measurements line.
		indexStart = indexStart - 1;
		indexStop = indexStop + 1;
	} else { //-- There are more than one points in the current view. Add one point on each side, if they are there.
		
		if (indexStart > 0) {
			indexStart = indexStart - 1;
		}
		
		if (indexStop + 1 < this.data.length) {
			indexStop = indexStop + 1;
		}
		
	}

	//-- TODO: This is already calculated in lastCrossing.
	if ( indexStart === null && indexStop === this.data.length ) { //-- the user has scrolled further than the data points, so the trend line will be based on the last two data points.
		indexStart = this.data.length - 2;
		indexStop = this.data.length;
	}
	
	trendData = this.data.slice( indexStart, indexStop ); //-- part of the data which will be the basis for the calculation of the trendline

	if ( trendData.length <= 1 ) { //-- Do not draw the trend line when we do not have enough points.
		return null;
	}
	
	trendLineData = Model.getTrendLineData( trendData );
	
	//--TODO remove && this.settings.showGoal === true, should be part of the view, maybe.
	
	xCross = this.getXCross( trendLineData );
	
	if ( xCross ) {
		trendLineData.data.push( [xCross, this.goal] );
		trendLineData.xCross = xCross;
	} else { //-- no crossing of trend line and target line in the future.
		trendLineData.xCross = null;
	}
	
	return trendLineData;
		
};
/**
* Extends Model
*/
;var CompetitiveGraphModel = function ( options ) {
	
	this.name = 'CompetitiveGraphModel';
	
	this.options = options;
	
	this.data = {
		measurements: [],
		groupAverages: []
	}; //-- parsed data in Highcharts readable format.
	
	this.locale = null;
	
	this.totalNumberOfMeasurements = null;
	
	this.init();
	
};

extendConstructor( CompetitiveGraphModel, Model );


CompetitiveGraphModel.prototype.init = function () {
	
	this.uber.init.apply( this ); //-- Call the init of the parent in the context of this class.
	
	//-- Set the necessary variables
	this.locale = this.options.data.locale;

	this.totalNumberOfMeasurements = this.data.measurements.length + this.data.groupAverages.length;
	
	//-- Set the first and the last values.
	if ( this.data.measurements.length > 0 && this.data.groupAverages.length > 0 ) {
		this.firstXValue = [this.data.measurements[0][0], this.data.groupAverages[0][0]].min();
		this.lastXValue = [this.data.measurements.lastValue()[0], this.data.groupAverages.lastValue()[0]].max();
	} else if ( this.data.measurements.length > 0 && this.data.groupAverages.length === 0 ) {

		this.firstXValue = this.data.measurements[0][0];
		this.lastXValue = this.data.measurements.lastValue()[0];
	} else if ( this.data.measurements.length === 0 && this.data.groupAverages.length > 0 ) {
		this.firstXValue = this.data.groupAverages[0][0];
		this.lastXValue = this.data.groupAverages.lastValue()[0];
	} 
	
};

CompetitiveGraphModel.prototype.parseJSON = function () {
	var i, //-- Counter
	measurement,
	groupAverage,
	dateArray;
	
	//-- Loop trough the measurements and parse them in the for Highcharts readable format.
	for (i = 0; i < this.options.data.measurements.length; i++) {
		
		measurement = []; //-- temporary helper array.
		dateArray = this.options.data.measurements[i].created_on.split('/');
		
		measurement.push( Date.UTC( dateArray[0], ( dateArray[1] - 1 ), dateArray[2]) ); //-- Convert the date to a Unix date in milliseconds.
		measurement.push( this.options.data.measurements[i].value );
		
		this.data.measurements.push( measurement );
		
	}
	
	//-- Loop trough the group averages and parse them in the for Highcharts readable format.
	for (i = 0; i < this.options.data.group_averages.length; i++) {
		
		groupAverage = []; //-- temporary helper array.
		dateArray = this.options.data.group_averages[i].created_on.split('/');
		
		groupAverage.push( Date.UTC( dateArray[0], ( dateArray[1] - 1 ), dateArray[2]) ); //-- Convert the date to a Unix date in milliseconds.
		groupAverage.push( this.options.data.group_averages[i].value );
		
		this.data.groupAverages.push( groupAverage );
		
	}

};/**
* Parent constructor for the Charts Controllers. Meant to be extended through the extendConstructor function
*/
;var ChartController = function () {
	this.name = 'ChartController';
};

ChartController.prototype.init = function () {

	this.chart = null; //-- Reference to the chart object, containing the Highcharts graphic.
	this.dateSelector = null; //-- Reference to the date selector object.
	this.touchElement = null; //-- Element necessary for Hammer to register touch moves.
	this.rangeSlider = null; //-- Reference to slider object for zooming.
	this.model = null;
	this.pinchZoom = this.options.pinchZoom === false ? false : true;
	this.touchDrag = this.options.touchDrag === false ? false : true;
	this.mouseDrag = this.options.mouseDrag === false ? false : true;
	
	this.prepare();
	
	this.initModel();
	
	this.drawChart(); //-- Draw the chart, create the chart object.

	this.highchartsContainer = this.options.graphContainer.select('.highcharts-container')[0]; //-- Element used for catching events, like touch events, drag events, etc.
	
	
	if (this.options.selectElement) {
		//-- Instantiate the date selector.
		this.dateSelector = new DateSelector({
			$element: this.options.selectElement, //-- Html select element
			period: this.getPeriod(), //-- bandwidth of the measurement data. Is null if there are no measurement points
			firstSelectedOption: parseInt( this.options.dataContainer.getAttribute( 'data-time-frame' ) ), //-- Options that will be selected, set by clients settings.
			texts: this.options.dateSelectorTexts, //-- Necessary text object, containing the text and translation for 'custom selection'.
			locale: this.model.locale || 'en' //-- English is default
		});

		this.attachDateSelectorEvents();

	}
	
	this.setMaxScrollingDate();

	if (this.options.rangeSliderContainer) {
		//-- initiate the range slider
		this.rangeSlider = new RangeSlider({
			handle: this.options.rangeSliderHandle, //-- Html element
			track: this.options.rangeSliderTrack, //-- Html element
			range: this.options.rangeSliderRange, 
			container: this.options.rangeSliderContainer
		});

		this.attachRangeSliderEvents();	
		
	}

	if (this.pinchZoom === true) {
		this.attachTouchZoomEvents();
	}
	
	
	if (this.touchDrag === true || this.mouseDrag === true) {
		this.attachDragEvents();
	}
	
	

	this.attachMouseEvents();
	
};

/**
* If there was already another graph, destroy that and the other elements. 
* Note: this is not necessary, because a select of the graph reloads the page
*/
ChartController.prototype.prepare = function () {
	
	if ( this.chart ) {
		this.chart.graphic.destroy();
		delete this.chart;
		this.chart = null;
	}
	
	if ( this.dateSelector ) {

		this.dateSelector.destroy();
		delete this.dateSelector;
		this.dateSelector = null;
	}

	if ( this.rangeSlider ) {
		this.rangeSlider.destroy();
		delete this.rangeSlider;
		this.rangeSlider = null;
	}
	
	if ( this.model ) {
		delete this.model;
		this.model = null;
		
	}
	
};

ChartController.prototype.initModel = function () {}; //-- to be overwritten by child class

ChartController.prototype.drawChart = function () {}; //-- to be overwritten by child class

ChartController.prototype.setMaxScrollingDate = function () {}; //-- to be overwritten by child class

ChartController.prototype.attachMouseEvents = function () {
	var that = this;
		
	if ( this.chart.overflow === false ) {
		return;
	}

	//-- hover states
	this.highchartsContainer.observe('mouseover', function () {
		if (that.chart.overflow === false) {
			return;
		}
		that.hover(true);
	});
	

	this.highchartsContainer.observe('mouseout', function () {
		if (that.chart.overflow === false) {
			return;
		}
		that.hover(false);
	});

	//-- Mousedown looks like dragging
	this.highchartsContainer.observe('mousedown', function () {
		if (that.chart.overflow === false) {
			return;
		}
		this.addClassName('dragging');
		that.hover(false);
	});
	
};

ChartController.prototype.attachRangeSliderEvents = function () {
	
	var that = this;

	if ( this.chart.overflow === false ) {
		return;
	}
	
	/* Gui zoom */
	if ( this.rangeSlider ) {
		
		//-- Set callback for when the slider changes
		this.options.rangeSliderTrack.observe('rangeSlider:onSlide', function ( event ) {
			if ( that.sliding === false ) { //-- Do this only once.
				that.dateSelector && that.dateSelector.handleCustomSelectionOption( 'show' ); //-- show 'aangepaste selectie' in de date dropdown
				that.chart.setDataOnActionStart();

				that.sliding = true;
			}
			
			that.chart.guiZoom( event, that.options.rangeSliderRange );

		});
		
		//-- After slider has changed
		this.options.rangeSliderTrack.observe('rangeSlider:onChange', function ( event ) {
			if ( that.noRangeSliderChange === true ) { //-- Because the range slider is set on a date selector change, this event is fired. Here it will be catched.
				that.noRangeSliderChange = false;
				return;
			} else {
				that.sliding = false; //-- reset the sliding variable.
				that.chart.guiZoom( event, that.options.rangeSliderRange );
				that.dateSelector && that.dateSelector.handleCustomSelectionOption('show'); //-- show 'aangepaste selectie' in de date dropdown
				//-- Redraw the trendline
				//-- TODO: move this to the SingleMeasurementChartController
				if (typeof that.redrawTrendLine === 'function') {
					that.redrawTrendLine();
				}
			}

		});
		
	}
	
};

ChartController.prototype.attachTouchZoomEvents = function () {

	var that = this;

	if ( this.chart.overflow === false ) {
		return;
	}
	
	if (this.touchElement === null) {
		this.touchElement = new Hammer( this.highchartsContainer ); //-- create a Hammer touch element if it does not exist yet
	}
	
	
	if (this.model.totalNumberOfMeasurements > 1 ) {

		/* pinch zoom (touch) */
		this.touchElement.ontransformstart = function () {
			that.dateSelector && that.dateSelector.handleCustomSelectionOption('show');
			that.chart.setDataTouchZoomStart();
			that.chart.setDataOnActionStart();
		};

		this.touchElement.ontransform = function ( transformEvent ) {
			that.chart.touchZoom( transformEvent );
		};

		this.touchElement.ontransformend = function () {
			that.chart.touchZoomEnd();
			//-- TODO: move this to SingleMeasurement
			if (typeof that.redrawTrendLine === 'function') {
				that.redrawTrendLine();
			}
		}

	}
	
};

ChartController.prototype.attachDragEvents = function () {

	var that = this;
	
	if (this.touchElement === null) {
		this.touchElement = new Hammer( this.highchartsContainer ); //-- create a Hammer touch element if it does not exist yet
	}
	
	/* touch and mouse dragging */
	if ( this.model.totalNumberOfMeasurements > 1 ) {
		this.touchElement.ondragstart = function ( dragStartEvent ) {
			
			//-- Disable mouse dragging if necessary
			if (that.mouseDrag === false && dragStartEvent.originalEvent.type === 'mousemove') {
				return;
			}
			
			//-- Disable touch dragging if necessary
			if (that.touchDrag === false && dragStartEvent.originalEvent.type === 'touchmove') {
				return;
			}
			
			if (that.chart.overflow === false) {
				return;
			}
			
			that.chart.setDataDragStart( dragStartEvent );
			that.chart.setDataOnActionStart();
			that.highchartsContainer.addClassName('dragging');
		};

		this.touchElement.ondrag = function ( dragEvent ) {

			//-- Disable mouse dragging if necessary
			if (that.mouseDrag === false && dragEvent.originalEvent.type === 'mousemove') {
				return;
			}

			//-- Disable touch dragging if necessary
			if (that.touchDrag === false && dragEvent.originalEvent.type === 'touchmove') {
				return;
			}

			if (that.chart.overflow === false) {
				return;
			}
			that.chart.drag( dragEvent );
		};

		this.touchElement.ondragend = function ( dragendEvent ) {

			//-- Disable mouse dragging if necessary
			if (that.mouseDrag === false && dragendEvent.originalEvent.type === 'mousemove') {
				return;
			}

			//-- Disable touch dragging if necessary
			if (that.touchDrag === false && dragendEvent.originalEvent.type === 'touchend') {
				return;
			}

			that.highchartsContainer.removeClassName('dragging');
			that.chart.dragEnd();
			
			//-- TODO: move this to SingleMeasurement
			if (typeof that.redrawTrendLine === 'function') {
				that.redrawTrendLine();
			}
			if (typeof that.maybeRedrawTargetLine === 'function') {
				that.maybeRedrawTargetLine();
			}
		};

		this.touchElement.onswipe = function ( swipeEvent) {

			//-- Disable mouse dragging if necessary
			if (that.mouseDrag === false && swipeEvent.originalEvent.type === 'touchend') {
				return;
			}

			//-- Disable touch dragging if necessary
			if (that.touchDrag === false && swipeEvent.originalEvent.type === 'touchend') {
				return;
			}

			// var trendLineData = that.model.getTrendLineData( that.chart.graphic.xAxis[0].getExtremes() ); //-- REMOVED this 24-6-2013, can probably be removed permanently
			that.chart.dragEnd();

			//-- TODO: move this to SingleMeasurement
			if (typeof that.redrawTrendLine === 'function') {
				that.redrawTrendLine();
			}
			if (typeof that.maybeRedrawTargetLine === 'function') {
				that.maybeRedrawTargetLine();
			}
		};

		this.chart.$container.observe('chart:onDrag', function () {
			that.dateSelector && that.dateSelector.handleCustomSelectionOption('show'); //-- show 'aangepaste selectie' in de date dropdown
		});
	
	}
	
};


ChartController.prototype.attachDateSelectorEvents = function () {};
/**
* Chart controller for a single measurement graphic.
* Extends ChartController
*/
;var SingleMeasurementChartController = function ( options ) {
	this.name = 'SingleMeasurementChartController';

	this.sliding = false; //-- Used for executing a block of code only once.

	this.settings = { //-- Default settings, to be overwritten by user settings
		showGoal: true,
		showValue: true,
		showTrend: true
	};

	this.noRangeSliderChange = false; //-- Boolean, helper.

	this.json = options.json;
	
	this.options = options.options;
	
	/**
	* Sets the hover class to the chart container for styling purposes (cursor: pointer).
	*/
	/* TODO: possible to move this to the parent? */
	this.hover = (function () {

		var hover = false;

		return function ( bool ) {

			if (bool) {
				if (hover) {
					return;
				} else {
					this.highchartsContainer.addClassName('hover');
					hover = true;
				}
			} else {
				if (!hover) {
					return;
				} else {
					this.highchartsContainer.removeClassName('hover');
					hover = false;
				}

			}
		};
	})();
	
	this.init();
	
};

extendConstructor(SingleMeasurementChartController, ChartController);

/**
* Constructor
*/
SingleMeasurementChartController.prototype.init = function () {
	var startDate;
	
	this.updateSettings();
	
	this.uber.init.apply( this ); //-- Call the init of the parent in the context of this class.
	
	this.attachEvents();

	//-- Update the view according to the selected date period
	if (this.model.data.length > 1 ) {
		
		if ( this.dateSelector !== null ) {
			//-- Date selector dropdown is activated
			startDate = this.model.data.lastValue()[0] - DateSelector.dtm(this.dateSelector.$element.getValue());
		} else {
			//-- There is no date selector dropdown
			startDate = this.model.data.lastValue()[0] - DateSelector.dtm(this.options.dataContainer.getAttribute('data-time-frame'));
			
			//-- If the desired start date is before the first measurement point, then the start date will be the first measurement point.
			if (startDate < this.model.data[0][0] ) {
				startDate = this.model.data[0][0];
			}
			
		}

		this.doInitZoom( startDate );
		
		if (this.rangeSlider) {
			this.noRangeSliderChange = true; //-- prevent action on the onchange event that will be fired of the range slider
			this.rangeSlider.slider.setValue( this.chart.getZoomLevel( this.options.rangeSliderRange ) ); //-- Set the init level of the range slider
			this.noRangeSliderChange = false;
		}
		
	} else {
		this.chart.overflow = false;
	}

	//-- Disable zooming with the range slider if we have 0 or 1 measurements.
	if ( this.rangeSlider && ( this.model.data.length < 2 || this.chart.overflow === false ) ) {
		this.rangeSlider.disable();
	}
	
};

/**
* Updates which line should be drawn in the graphic. Reads from data-attributes set by Rails. Overrides default settings object.
*/
SingleMeasurementChartController.prototype.updateSettings = function () {
	
	if ( this.options.dataContainer.getAttribute('data-line-goal') === 'false' ) {
		this.settings.showGoal = false;
	}

	if ( this.options.dataContainer.getAttribute('data-line-type') === 'false' ) {
		this.settings.showValue = false;
	}

	if ( this.options.dataContainer.getAttribute('data-line-lin') === 'false' ) {
		this.settings.showTrend = false;
	}
	
};

SingleMeasurementChartController.prototype.initModel = function () {
	
	this.model = new SingleMeasurementModel({
		data: this.json
	});


	
};

SingleMeasurementChartController.prototype.setMaxScrollingDate = function () {

	this.chart.setMaxScrollingDate( this.model.lastCrossing ); //-- No scrolling beyond the last crossing date

};

/**
* Attach events specific for this type of graph.
*/
SingleMeasurementChartController.prototype.attachEvents = function () {
	
	var that = this;
	
	this.chart.$container.observe( 'chart:legendShowTrendClick', function () {
		that.redrawTrendLine();
	});
	
	this.chart.$container.observe( 'chart:legendTargetClick', function () {
		that.redrawTrendLine();
	});
	
		
};

/* TODO: move this to the model */
SingleMeasurementChartController.prototype.getPeriod = function () {
	var period;
	
	if ( this.model.data.length > 0 ) {
		period = [ this.model.data[0][0], this.model.lastXValue ];
	} else {
		period = null;
	}
	
	return period;
}

/**
* Creates a chart object, containing the Highcharts graphic, passing options.
*/
SingleMeasurementChartController.prototype.drawChart = function () {
	this.chart = new SingleMeasurementChart({
		data: this.model.data,
		firstXValue: this.model.firstXValue,
		lastXValue: this.model.lastXValue,
		legendText: this.json.label, //-- legend text for measurements
		container: this.options.chartPlaceholder, //-- Html element that will contain the graphic.
		locale: this.model.locale,
		goal: this.model.goal,
		units: this.json.units || '', //-- units of the measurements
		settings: this.settings, //-- user settings telling which line will be visible.
		finishSymbol: this.options.finishSymbol, //-- Image placed when trend line and target line cross.
		legend: this.options.legend === false ? false : true,
		tooltip: this.options.tooltip
	});

};
	
SingleMeasurementChartController.prototype.redrawTrendLine = function ( trendLineData ) {
	trendLineData = trendLineData || this.model.getTrendLineData( this.chart.graphic.xAxis[0].getExtremes() );
	if ( trendLineData ) {
		this.chart.drawTrendLine( trendLineData );
	}
};


/**
* Only called on a change of the date range selector, or to get the first view after page load, so that one of the standard settings are shown.
* If necessary redraws all lines: value, target and trend.
*/
SingleMeasurementChartController.prototype.doInitZoom = function ( startDate ) {
	
	var trendLineData,
	zoom;
	
	startDate = parseInt( startDate ); //-- Just to be sure we have an integer.
	
	if ( startDate ) {
		zoom = {
			min: startDate,
			max: null
		}
	} else { //-- startDate isNaN (all data)

		zoom = {
			min: this.model.data[0][0],
			max: null
		}
		
	}
	
	trendLineData = this.model.getTrendLineData( zoom ); //-- recalculate the trendline
	
	this.chart.doInitZoom( trendLineData, startDate);
	
};

SingleMeasurementChartController.prototype.attachDateSelectorEvents = function () {

	var that = this;
	
	if ( this.chart.overflow === false ) {
		return;
	}
	
	//-- Change of period selection
	this.dateSelector.$element.observe('change', function () {
		var startDate;
		
		that.noRangeSliderChange = true; //-- This is a genuine date selector change
		
		//-- Calculate the start date for displaying
		if ( DateSelector.dtm(parseInt( this.getValue() )) === 0 ) {
			startDate = 'all_dates';
		} else {
			startDate = that.model.lastXValue - DateSelector.dtm(this.getValue());
		}
		
		that.doInitZoom( startDate ); //-- Zoom all visible line according to the date range selected.
		that.rangeSlider && that.rangeSlider.slider.setValue ( that.chart.getZoomLevel( that.options.rangeSliderRange ) ); //-- The date range change brings also a change in the zoom level, so we have to show this in de range slider

		that.dateSelector.handleCustomSelectionOption('hide'); //-- This is not a custom selection, so this option in the drop down must be hidden.

		that.noRangeSliderChange = false; //-- Reset this flag.

	});
	
};
/**
* Chart controller for the competive graph.
* Extends ChartController
*/
;var CompetitiveGraphChartController = function ( options ) {

	this.name = 'CompetitiveGraphChartController';

	this.sliding = false; //-- Used for executing a block of code only once.

	this.noRangeSliderChange = false; //-- Boolean, helper.

	this.json = options.json;
	
	this.options = options.options;
	
	/**
	* Sets the hover class to the chart container for styling purposes (cursor: pointer).
	*/
	/* TODO: possible to move this to the parent? */
	this.hover = (function () {

		var hover = false;

		return function ( bool ) {

			if (bool) {
				if (hover) {
					return;
				} else {
					this.highchartsContainer.addClassName('hover');
					hover = true;
				}
			} else {
				if (!hover) {
					return;
				} else {
					this.highchartsContainer.removeClassName('hover');
					hover = false;
				}

			}
		};
	})();
	
	this.init();
	
};

extendConstructor(CompetitiveGraphChartController, ChartController);

/**
* Constructor
*/
CompetitiveGraphChartController.prototype.init = function () {
	var startDate,
	endDate;

	this.uber.init.apply( this ); //-- Call the init of the parent in the context of this class.

	//-- Update the view according to the selected date period
	if (this.model.data.measurements.length > 1 || this.model.data.groupAverages.length > 1 ) {

		if ( this.dateSelector !== null ) {
			
			//-- Date selector dropdown is activated
			startDate = this.model.lastXValue - DateSelector.dtm(this.dateSelector.$element.getValue());
			
		} else if ( this.options.dataContainer.getAttribute('data-time-frame') !== null ){
			
			//-- There is no date selector dropdown
			startDate = this.model.lastXValue - DateSelector.dtm(this.options.dataContainer.getAttribute('data-time-frame'));
			
			//-- If the desired start date is before the first measurement point, then the start date will be the first measurement point.
			if (startDate < this.model.firstXValue ) {
				startDate = this.model.firstXValue;
			}
			
		} else { //-- no dropdown for period selection, no timeframe. 

			startDate = this.model.firstXValue;
			
		}

		this.doInitZoom( startDate, this.model.lastXValue );



		if (this.rangeSlider) {
			this.noRangeSliderChange = true; //-- prevent action on the onchange event that will be fired of the range slider
			this.rangeSlider.slider.setValue( this.chart.getZoomLevel( this.options.rangeSliderRange ) ); //-- Set the init level of the range slider
			this.noRangeSliderChange = false;
		}

	} else {
		this.chart.overflow = false;
	}
	
	//-- Disable zooming with the range slider if we have 0 or 1 measurements.
	if ( this.rangeSlider && (this.model.totalNumberOfMeasurements < 2 || this.chart.overflow === false) ) {
		this.rangeSlider.disable();
	}
	

	
};

CompetitiveGraphChartController.prototype.initModel = function () {

	this.model = new CompetitiveGraphModel({
		data: this.json
	});
	
};

/* TODO: move this to the model */
CompetitiveGraphChartController.prototype.getPeriod = function () {
	var period;
	
	if ( this.model.data.measurements.length > 0 || this.model.data.groupAverages.length > 0 ) {
		period = [ this.model.firstXValue, this.model.lastXValue ];
	} else {
		period = null;
	}
	
	return period;
	
};


/**
* Only called on a change of the date range selector, or to get the first view after page load, so that one of the standard settings are shown.
* If necessary redraws all lines: value, target and trend.
*/
CompetitiveGraphChartController.prototype.doInitZoom = function ( startDate, endDate ) {
	
	startDate = parseInt( startDate ); //-- Just to be sure we have an integer.
	endDate = parseInt( endDate ); //-- Just to be sure we have an integer.
	
	this.chart.doInitZoom( startDate, endDate );
	
};

/**
* Creates a chart object, containing the Highcharts graphic, passing the data from the model and options.
*/
CompetitiveGraphChartController.prototype.drawChart = function () {

	this.chart = new CompetitiveGraphChart({
		data: this.model.data,
		firstXValue: this.model.firstXValue,
		lastXValue: this.model.lastXValue,
		period: this.getPeriod(),
		legendText: this.json.label_user, //-- legend text for measurements
		legendGroupText: this.json.label_group, //-- legend text for measurements
		container: this.options.chartPlaceholder, //-- Html element that will contain the graphic.
		locale: this.model.locale || 'en', //-- If the json file contains no locale, English is the default.
		units: this.json.units || '', //-- units of the measurements
		legend: this.options.legend === false ? false : true,
		tooltip: this.options.tooltip
	});
	
};

CompetitiveGraphChartController.prototype.attachDateSelectorEvents = function () {

	var that = this;
	
	if ( this.chart.overflow === false ) {
		return;
	}
	
	//-- Change of period selection
	this.dateSelector.$element.observe('change', function () {
		var startDate;
		
		that.noRangeSliderChange = true; //-- This is a genuine date selector change
		
		//-- Calculate the start date for displaying
		if ( parseInt( this.getValue() ) === 0 ) {
			startDate = 'all_dates';
		} else {
			startDate = that.model.lastXValue - DateSelector.dtm(this.getValue());
		}
		
		that.doInitZoom( startDate, that.model.lastXValue ); //-- Zoom all visible line according to the date range selected.
		that.rangeSlider && that.rangeSlider.slider.setValue ( that.chart.getZoomLevel( that.options.rangeSliderRange ) ); //-- The date range change brings also a change in the zoom level, so we have to show this in de range slider

		that.dateSelector.handleCustomSelectionOption('hide'); //-- This is not a custom selection, so this option in the drop down must be hidden.

		that.noRangeSliderChange = false; //-- Reset this flag.

	});
	
};

CompetitiveGraphChartController.prototype.setMaxScrollingDate = function () {
	return this.model.lastXValue;
	
};;var AppController = function ( options ) {

	var locale = options.dataContainer.getAttribute('data-locale'),
	jsonUrl = options.dataContainer.getAttribute('data-url'); //-- json url

	this.json = null; //-- Raw json from the server containing the clients measurements.
	
	this.options = options; //-- Object. Options passed as arguments to this constructor.

	this.loadJSON( jsonUrl );
	
};

/**
* JSON is loaded. Start the drawing of the chart.
*/
AppController.prototype.initApp = function () {
	if ( this.options.dataContainer.getAttribute( 'data-show-group' )  === 'true' ) {
		new CompetitiveGraphChartController({
			json: this.json.result,
			options: this.options
		});
	} else {
		new SingleMeasurementChartController({
			json: this.json.result,
			options: this.options
		});
	}
};
	

/**
* Loads the json from the server
*/
AppController.prototype.loadJSON =function (jsonUrl) {
	var that = this;
	
	//-- Nice to have: show an error message if the data could not be loaded.

	new Ajax.Request(jsonUrl, {
		method: 'get',
		onSuccess: function( transport ){
			that.json = transport.responseText.evalJSON();
			that.initApp(); //-- This fires the drawing of the graphic.
		}
	});

};

//document.observe('dom:loaded', function () {

document.observe('dom:loaded', kickOffGraph);

function kickOffGraph() {

	//-- Graph in My graphs
	if ( $$('body.measurements:not(body.measurements-show_large)')[0] ) {
		
		var graphContainer = $$('.highcharts-graph')[0];
		
		if ( graphContainer ) {

			new AppController({
				graphContainer: graphContainer, //-- outer most container, containing the date selector, zoom slider and graphic
				chartPlaceholder: graphContainer.select('.chart-placeholder')[0], // inner container for only the graph
				finishSymbol: 'url(/images/measurements/finish.png)', //-- TODO: only include the path and do the url thing in chart.
				rangeSliderHandle: graphContainer.select( '.handle' )[0],
				rangeSliderTrack: graphContainer.select( '.track' )[0],
				rangeSliderContainer: graphContainer.select( '.range-slider-container' )[0],
				rangeSliderRange: 40,
				selectElement: graphContainer.select( '.date-select' )[0], //-- html select element to select the date period
				dataContainer: $('measurements-layout'), //-- the html element containing the data attributes with the user settings
				dateSelectorTexts: {
					customSelection: {
						en: 'custom selection',
						nl: 'aangepaste selectie'
					}
				}
			});
			
		}

	}
	
	//-- Graphs in My Profile section
	if ( $$( 'body.users-graphs' )[0] ) {
		
		$$('.graphs.section .graph').each(function (element) {
			

			var graphContainer = element.select('.highcharts-graph')[0];
			
			new AppController({
				graphContainer: graphContainer, //-- outer most container, containing the date selector, zoom slider and graphic
				chartPlaceholder: graphContainer.select('.chart-placeholder')[0], // inner container for only the graph
				finishSymbol: 'url(/images/measurements/finish.png)', //-- TODO: only include the path and do the url thing in chart.
				rangeSliderHandle: graphContainer.select( '.handle' )[0],
				rangeSliderTrack: graphContainer.select( '.track' )[0],
				rangeSliderContainer: graphContainer.select( '.range-slider-container' )[0],
				rangeSliderRange: 40,
				selectElement: graphContainer.select( '.date-select' )[0], //-- html select element to select the date period
				dataContainer: element, //-- the html element containing the data attributes with the user settings
				dateSelectorTexts: {
					customSelection: {
						en: 'custom selection',
						nl: 'aangepaste selectie'
					}
				}
			});
			
		});
		
	} 

	//-- Graphs in the admin section
	if ( $$( 'body.admin-bio_feedback' )[0] ) {
		
		var graphContainer = $$('#graph .highcharts-graph')[0];
		
		new AppController({
			graphContainer: graphContainer, //-- outer most container, containing the date selector, zoom slider and graphic
			chartPlaceholder: graphContainer.select('.chart-placeholder')[0], // inner container for only the graph
			finishSymbol: 'url(/images/measurements/finish.png)', //-- TODO: only include the path and do the url thing in chart.
			rangeSliderHandle: graphContainer.select( '.handle' )[0],
			rangeSliderTrack: graphContainer.select( '.track' )[0],
			rangeSliderContainer: graphContainer.select( '.range-slider-container' )[0],
			rangeSliderRange: 40,
			selectElement: graphContainer.select( '.date-select' )[0], //-- html select element to select the date period
			dataContainer: $('graph-container'), //-- the html element containing the data attributes with the user settings
			dateSelectorTexts: {
				customSelection: {
					en: 'custom selection',
					nl: 'aangepaste selectie'
				}
			}
		});
		
	}
	
	//-- Enlarged graph
	if ( $$('body.measurements-show_large')[0] ) {
		
		var graphContainer = $$('.highcharts-graph')[0];
		
		if ( graphContainer ) {
			
			new AppController({
				graphContainer: graphContainer, //-- outer most container, containing the date selector, zoom slider and graphic
				chartPlaceholder: graphContainer.select('.chart-placeholder')[0], // inner container for only the graph
				finishSymbol: 'url(/images/measurements/finish.png)', //-- TODO: only include the path and do the url thing in chart.
				rangeSliderHandle: graphContainer.select( '.handle' )[0],
				rangeSliderTrack: graphContainer.select( '.track' )[0],
				rangeSliderContainer: graphContainer.select( '.range-slider-container' )[0],
				rangeSliderRange: 40,
				selectElement: graphContainer.select( '.date-select' )[0], //-- html select element to select the date period
				dataContainer: $('measurements-layout'), //-- the html element containing the data attributes with the user settings
				dateSelectorTexts: {
					customSelection: {
						en: 'custom selection',
						nl: 'aangepaste selectie'
					}
				}
			});
			
		}

	}
	
	//-- App graph
	if ( $$('div.app-graph')[0] ) {

		var graphContainer = $$('.highcharts-graph')[0];
		
		if ( graphContainer ) {
			new AppController({
				legend: false,
				tooltip: false,
				pinchZoom: false,
				touchDrag: false,
			  mouseDrag: false,
				graphContainer: graphContainer, //-- outer most container, containing the date selector, zoom slider and graphic
				chartPlaceholder: graphContainer.select('.chart-placeholder')[0], // inner container for only the graph
        finishSymbol: 'url(grafiek/img/finish.png)', //-- TODO: only include the path and do the url thing in chart.
        rangeSliderHandle: graphContainer.select( '.handle' )[0],
				rangeSliderTrack: graphContainer.select( '.track' )[0],
				rangeSliderContainer: graphContainer.select( '.range-slider-container' )[0],
				rangeSliderRange: 40,
				selectElement: graphContainer.select( '.date-select' )[0], //-- html select element to select the date period
				dataContainer: $('measurements-layout'), //-- the html element containing the data attributes with the user settings
				dateSelectorTexts: {
					customSelection: {
						en: 'custom selection',
						nl: 'aangepaste selectie'
					}
				}
			});
			
		}
	}
	
	//-- Competive graph (Groepsgrafieken)
	
	if ( $$( 'body.competitive_graph' )[0] ) {
		
		var graphContainer = $$('#graph .highcharts-graph')[0];
		
		new AppController({
			graphContainer: graphContainer, //-- outer most container, containing the date selector, zoom slider and graphic
			chartPlaceholder: graphContainer.select('.chart-placeholder')[0], // inner container for only the graph
			rangeSliderHandle: graphContainer.select( '.handle' )[0],
			rangeSliderTrack: graphContainer.select( '.track' )[0],
			rangeSliderContainer: graphContainer.select( '.range-slider-container' )[0],
			rangeSliderRange: 40,
			selectElement: graphContainer.select( '.date-select' )[0], //-- html select element to select the date period
			dataContainer: $('graph-container'), //-- the html element containing the data attributes with the user settings
			dateSelectorTexts: {
				customSelection: {
					en: 'custom selection',
					nl: 'aangepaste selectie'
				}
			}
		});
		
	}
	
	
};
//})(this);
