"use strict";
var $j = jQuery;

var textarea_height = 100;
var MAX_TOPICS_PER_FETCH = 15;
var refreshHome = true;
var refreshForumIndex = false;
var openPanelID = null;

var month_01 = 'jan';
var month_02 = 'feb';
var month_03 = 'maa';
var month_04 = 'apr';
var month_05 = 'mei';
var month_06 = 'jun';
var month_07 = 'jul';
var month_08 = 'aug';
var month_09 = 'sep';
var month_10 = 'okt';
var month_11 = 'nov';
var month_12 = 'dec';

var full_month_1 = 'januari';
var full_month_2 = 'februari';
var full_month_3 = 'maart';
var full_month_4 = 'april';
var full_month_5 = 'mei';
var full_month_6 = 'juni';
var full_month_7 = 'juli';
var full_month_8 = 'augustus';
var full_month_9 = 'september';
var full_month_10 = 'oktober';
var full_month_11 = 'november';
var full_month_12 = 'december';

var day_0 = 'zondag';
var day_1 = 'maandag';
var day_2 = 'dinsdag';
var day_3 = 'woensdag';
var day_4 = 'donderdag';
var day_5 = 'vrijdag';
var day_6 = 'zaterdag';

// generic catch for errors
window.onerror = function (o, file, line) {
  var cache = [];
  console.log('Error gevangen: ' + file + ':' + line + '\n' + JSON.stringify(o, function (key, value) {
    if (typeof value === 'object' && value !== null) {
      if (cache.indexOf(value) !== -1) {
        // Circular reference found, discard key
        return;
      }
      // Store value in our collection
      cache.push(value);
    }
    return value;
  }));
  cache = null; // preserving memory
};

// Make Jquery contains case-insensitive
jQuery.expr[":"].Contains = jQuery.expr.createPseudo(function(arg) {
  return function( elem ) {
    return jQuery(elem).text().toUpperCase().indexOf(arg.toUpperCase()) >= 0;
  };
});

function isAndroid() {
  return navigator.userAgent.toLowerCase().indexOf("android") > -1;
}

function isAndroid2() {
  return navigator.userAgent.toLowerCase().indexOf("android 2") > -1;
}

function isIOS() {
  return navigator.userAgent.match(/(iPad|iPhone|iPod)/i);
}

function isMobile() {
  return isIOS() || isAndroid();
}

var sound_game_link_click;
var sound_game_button_click;
var sound_game_reward;

function onDeviceReady() {
  document.addEventListener("resume", onResume, false);
  document.addEventListener("backbutton", onBackButtonPressed, false);

  var prefix = isAndroid() ? '/android_asset/www/' : '';
  sound_game_link_click = new Media(prefix + 'sound/game/link-click.mp3');
  sound_game_button_click = new Media(prefix + 'sound/game/button-click.mp3');
  sound_game_reward = new Media(prefix + 'sound/game/reward.mp3');

  // avoid a white screen when the splashscreen is hidden on Android
  setTimeout(function() {
    navigator.splashscreen.hide();
  }, isAndroid() ? 1500 : 0);
}

// The Android backbutton should just do the same as clicking the backbutton in the header (with some added sugar for closing the menu)
function onBackButtonPressed() {
  if (openPanelID != null) {
    $j('#'+openPanelID).panel("close");
  } else {
    var currentPage = $j.mobile.activePage[0].id;
    if (currentPage == "homePage" || currentPage == "loginPage") {
      navigator.app.exitApp();
    } else {
      $j('#' + currentPage)
          .find('.backbutton')
          .trigger('click');
    }
  }
}

function onResume() {
  // refresh the homepage on resume
  refreshHome = true;
  // if we're currently on the homepage, make sure it's refreshed by triggering a transition
  if ($j.mobile.activePage[0].id == "homePage") {
    $j.mobile.changePage($j("#homePage"), {transition: "none", allowSamePageTransition: true});
  }
}

function getServiceURL(servicePath) {
  // on a device, connect to the testserver, otherwise to the dev machine
  var fullServicePath = "/" + appLanguage + "/api/" + apiVersion;
//  if (isMobile()) {
  if (true) {
    return apiUrl + fullServicePath + servicePath;
  } else {
    // strip any params from the path
    if (servicePath.indexOf('?') > -1) {
      servicePath = servicePath.substring(0, servicePath.indexOf('?'));
    }
    return "serverstub" + fullServicePath + servicePath + ".json";
  }
}

/**
 * Sommige Androidversies geven 'illegal date' voor new Date ("2013-02-20T14:49:27TZD")
 * Daarom eigen functie gemaakt 
 */
function getDate(dateString /* 2013-02-20T14:49:27TZD */) {
	var y = dateString.substring(0, 4);
  var m = dateString.substring(5, 7);
  var d = dateString.substring(8, 10);
  return new Date(y, m - 1, d);
}

/**
 * Indien vandaag: toon uren en minuten, bv '15:34'
 * Zo nee, indien gisteren: toon 'gisteren'
 * Zo nee, indien < 7 dagen geleden: toon de dag, bv 'donderdag'
 * Zo nee, indien dit jaar: toon de datum, bv '31 aug'
 * Zo nee: toon datum en jaar, bv '31 aug 2012'
 * @param dateString
 * @return {string}
 */
function getDateString(dateString /* 2013-02-20T14:49:27TZD */) {
  var compareDate = getToday();

	var y = dateString.substring(0, 4);
  var m = dateString.substring(5, 7);
  var d = dateString.substring(8, 10);

  var theDate = getDate(dateString);
  var theTime = theDate.getTime();

  // check for today
  if (theTime == compareDate.getTime()) {
    var h = dateString.substring(11, 13);
    var min = dateString.substring(14, 16);
    return h + ":" + min;
  }

  // check for yesterday
  compareDate.setDate(compareDate.getDate() - 1);
  if (theTime == compareDate.getTime()) {
    return "gisteren";
  }

  // check for this week
  compareDate.setDate(compareDate.getDate() - 6); // already subtracted one day
  if (theTime > compareDate.getTime()) {
    return getDayName(theDate.getDay());
  }

  // check for this year
  if (y == compareDate.getYear()) { // last weeks year is fine
    return d + " " + getMonthName(m);
  }

  // sometime before this year
  return d + " " + getMonthName(m) + " " + y;
}

function getMonthName(monthNumber) {
  return eval("month_" + monthNumber);
}

function getFullMonthName(monthNumber) {
  return eval("full_month_" + monthNumber);
}

function getDayName(dayNumber) {
  return eval("day_" + dayNumber);
}

function getToday() {
  var today = new Date();
  today.setHours(0);
  today.setMinutes(0);
  today.setSeconds(0);
  today.setMilliseconds(0);
  return today;
}

function getPersonPhotoUrl(url) {
  // TODO ask DJ to send 70x70 personimages to the client
  return url == null ? "img/app/avatar.png" : url;
}
function getUserProfileUrl (user) {
  var fromPageUrl = '#' + $j.mobile.activePage.attr('id');
  return '#userProfilePage?userId=' + user.id + '&fromPageUrl=' + escape(fromPageUrl);
}

function getFullName(user) {
  return user.firstname + ' ' + user.lastname;
}

function getUserImg (user) {
  return '<img class="user_thumb" onClick="document.location.href=\''+getUserProfileUrl(user)+'\'" src="' + getPersonPhotoUrl(user.photo_url) + '"/>'
}

function getRecipePhotoUrl(illustrations) {
  return illustrations.length == 0 ? "img/app/recipe_placeholder.png" : illustrations[0].photo_url;
}

function logout() {
  storeUser(null);
  setPassword(null);
}

function isLoggedIn() {
  return getUser() != null;
}

function configureSubscribeButton(element, url, isSubscribed) {
  $j(element)
      .val(isSubscribed ? 'Zeg e-mail abonnement op' : 'Abonneer per e-mail')
      .button('refresh')
      .unbind('click')
      .bind('click', function () {
        var that = $j(this);
        invokeRemote(
            isSubscribed ? 'DELETE' : 'POST',
            getServiceURL(url),
            null,
            true,
            function (data) {
              showAlert("Opslaan gelukt", "Je instelling is opgeslagen.");
              configureSubscribeButton(element, url, !isSubscribed);
            });
      });
}

function doPost(url, data, async, successFunction, errorFunction) {
  invokeRemote('POST', url, data, async, successFunction, errorFunction);
}

function doGet(url, async, successFunction, errorFunction) {
  invokeRemote('GET', url, null, async, successFunction, errorFunction);
}

function doDelete(url, async, successFunction, errorFunction) {
  invokeRemote('DELETE', url, null, async, successFunction, errorFunction);
}

function invokeRemote(method, url, data, async, successFunction, errorFunction) {
  var loginName = getLoginName();
  var password = getPassword();
  $j.ajax({
    type: method,
    headers: {
      "X-ECOACHING-EMAIL": btoa(loginName),
      "X-ECOACHING-PASSWORD": btoa(password)
    },
    url: url,
    cache:false,
    data: data,
    dataType: 'json',
    async: async,
    success: function (data) {
      successFunction(data);
    },
    error: function (XMLHttpRequest, textStatus, errorThrown) {
      if (errorFunction != null) {
        console.log("service failed: " + url + ", " + textStatus + "; " + errorThrown);
        errorFunction();
      } else {
//        showAlert("Melding", "Service failed: " + url + ", " + textStatus + "; " + errorThrown);
        console.log("service failed: " + url + ", " + textStatus + "; " + errorThrown);
        $j.mobile.changePage($j("#noNetworkPage"), {transition: "none"});
      }
    }
  });
}

function setLoginName(loginName) {
  localStorage.setItem("loginName", loginName);
}

function getLoginName() {
  return localStorage.getItem("loginName");
}

function setPassword(password) {
  localStorage.setItem("password", password);
}

function getPassword() {
  return localStorage.getItem("password");
}

function storeUser(obj) {
  return localStorage.setItem("user", JSON.stringify(obj));
}

function getUser() {
  var usr = localStorage.getItem("user");
  if (usr == null || usr == 'undefined') {
    return null;
  } else {
    return JSON.parse(usr);
  }
}

function back(toPageID) {
  $j.mobile.changePage(toPageID, {reverse: true});
}

function extractFilename(theURI) {
  return theURI.substring(theURI.lastIndexOf("/") + 1);
}

function rewriteAnchorsInsideElement(theElement) {
  $j(theElement)
      .find("a")
      .each(function (i, anchor) {
        $j(anchor)
            .attr('onclick', "openWindow(\'" + anchor.href + "\')") // links are rewritten to use the InAppBrowser API, via the openWindow function
            .attr('href', "#") // the href tag is replaced by a #
            .addClass('ui-link') // make sure the links have a nice look
        ;
      });
}

// not cool that the server sends inline scripts, so make sure they're disabled and also make sure the word 'script' can still be used
function deleteScript(content) {
  while (content.indexOf("script") > -1) {
    content = content.replace("script", "scr&shy;ipt");
  }
  return content;
}

function openWindow(pleaseTakeMeHere) {
  window.open(pleaseTakeMeHere, '_system');
}

function showAlert(title, txt) {
  if (isMobile()) {
    navigator.notification.alert(txt, function () {
    }, title);
  } else {
    alert(title + "\n\n" + txt);
  }
}

function showAlertWithCallback(title, txt, callback) {
  if (isMobile()) {
    navigator.notification.alert(txt, callback, title);
  } else {
    alert(title + "\n\n" + txt);
  }
}

function loadHomepageHeader() {
  var pageName = "homePage";
  var menuPanelID = pageName + "_menupanelid";

  $j('#' + pageName + " .header")
      .attr('data-role', 'header')
      .attr('data-tap-toggle', 'false')
      .attr('data-position', 'fixed')
      .html(('<div id="homePageLogoContainer"><img src="img/app/logo_header.png" height="30"/></div>') +
      ('<a id="menuButton" data-icon="bars" data-iconpos="notext" data-mini="true" href="#' + menuPanelID + '" data-position-to="window" class="ui-btn-right game-menu-icon"></a>') +
//        ('<div class="menuButtonContainer"><img src="../img/category.png" height="20px" data-menupanelid="'+menuPanelID+'" id="' + menuPanelID + '_button" class="menuPanelButton"/></div>') +
      '      <div class="subheader"></div>');

  addMenuToPage(pageName, menuPanelID);
}
// executed one time per page (pagebeforecreate)
function loadHeader(pageName, addMenu, h1, backHref) {
  var menuPanelID = pageName + "_menupanelid";

  $j('#' + pageName + " .header")
      .attr('data-role', 'header')
      .attr('data-tap-toggle', 'false')
      .attr('data-position', 'fixed')
      .html((!backHref ? '' : '  <a class="backbutton" data-mini="true" href="' + backHref + '" data-icon="arrow-l" data-direction="reverse" class="button_back">Terug</a>') +
          '  <h1>' + h1 + '</h1>' +
          (!addMenu ? '' : '<a id="menuButton" data-icon="bars" data-iconpos="notext" data-mini="true" href="#' + menuPanelID + '" data-position-to="window" class="ui-btn-right game-menu-icon"></a>') +
//          (!addMenu ? '' : '<div class="menuButtonContainer"><img src="../img/category.png" height="20px" data-menupanelid="'+menuPanelID+'" id="' + menuPanelID + '_button" class="menuPanelButton"/></div>') +
          '      <div class="subheader"></div>');

  if (addMenu) {
    addMenuToPage(pageName, menuPanelID);
  }
}

function loadGameHomepageHeader(pageName) {
  var menuPanelID = pageName + "_menupanelid";

  $j('#' + pageName + " .header")
      .attr('data-role', 'header')
      .attr('data-tap-toggle', 'false')
      .attr('data-position', 'fixed')
      .html(
          '  <a class="backbutton invisible" data-mini="true" href="#homePage" data-icon="arrow-l" data-direction="reverse" class="button_back">Terug</a>' +
          '<img src="img/game/game-logo.png" width="75%" style="margin-left: 10px; margin-top: 10px"/>' +
          '<a id="menuButton" data-icon="bars" data-iconpos="notext" data-mini="true" href="#' + menuPanelID + '" data-position-to="window" class="ui-btn-right game-menu-icon"></a>'
  );

  addMenuToPage(pageName, menuPanelID);
}

function loadGamePageHeader(pageName, h1, backHref) {
  var menuPanelID = pageName + "_menupanelid";

  $j('#' + pageName + " .header")
      .attr('data-role', 'header')
      .attr('data-tap-toggle', 'false')
      .attr('data-position', 'fixed')
      .html((!backHref ? '' : '<a class="backbutton" data-mini="true" href="' + backHref + '" data-icon="arrow-l" data-direction="reverse" class="button_back">Terug</a>') +
            '  <h1 id="'+pageName+'headerTitle">' + h1 + '</h1>' +
            '<a id="menuButton" data-icon="bars" data-iconpos="notext" data-mini="true" href="#' + menuPanelID + '" data-position-to="window" class="ui-btn-right game-menu-icon"></a>'
  );

  addMenuToPage(pageName, menuPanelID);
}

function addMenuToPage(pageName, menuPanelID) {
  $j('#' + pageName).append('<div id="'+menuPanelID+'" data-role="panel" data-theme="d" data-position="right" data-position-fixed="false" data-display="reveal" class="menupanel"></div>');

  var forumsMenuItem = (homepageElements.indexOf(HOMEPAGE_ELEMENT_FORUMS) == -1 ? '' :
      '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'forumOverviewPage' ? ' href="#forumOverviewPage"' : ' onclick="closePanel(\'' + menuPanelID + '\');"' ) + '>Forums</a></li>');

  var recipeMenuItem = (homepageElements.indexOf(HOMEPAGE_ELEMENT_RECIPE) == -1 ? '' :
      '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'recipeStartPage' ? ' href="#recipeStartPage"' : ' onclick="closePanel(\'' + menuPanelID + '\');"' ) + '>Recepten</a></li> ');

  var graphMenuItems = (homepageElements.indexOf(HOMEPAGE_ELEMENT_GRAPH) == -1 ? '' :
      (isAndroid2() ? '' : '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'weightGraphPage' ? ' href="#weightGraphPage"' : ' onclick="closePanel(\'' + menuPanelID + '\');"' ) + '>Gewichtsgrafiek</a></li>') +
      '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'addWeightPage' ? ' href="#addWeightPage"' : ' onclick="closePanel(\'' + menuPanelID + '\');"' ) + '>Voeg gewicht toe</a></li>' +
      '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'targetWeightPage' ? ' href="#targetWeightPage"' : ' onclick="closePanel(\'' + menuPanelID + '\');"' ) + '>Stel streefgewicht in</a></li>'
      );

  var gameMenuItems = (homepageElements.indexOf(HOMEPAGE_ELEMENT_GAME) == -1 ? '' :
      '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'gameHomePage' ? ' href="#gameHomePage?tab=today"' : ' onclick="closePanel(\'' + menuPanelID + '\');"') + '>Beweeg-game</a></li> ' +
      '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'gameAddActivityPage' ? ' href="#gameAddActivityPage"' : ' onclick="closePanel(\'' + menuPanelID + '\');"') + '>Voeg activiteit toe</a></li> ');

  var gameRuleItem = (homepageElements.indexOf(HOMEPAGE_ELEMENT_GAME) == -1 ? '' : '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'gameRulesPage' ? ' href="#gameRulesPage?tab=today"' : ' onclick="closePanel(\'' + menuPanelID + '\');"') + '>Spelregels - game</a></li> ');

  var faqItem = (!FAQ_ENABLED ? '' : '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'faqPage' ? ' href="#faqPage"' : ' onclick="closePanel(\'' + menuPanelID + '\');"') + '>Veelgestelde vragen</a></li> ');

  var menuItems = '';
  for (var i = 0; i < homepageElements.length; i++) {
    var part = homepageElements[i];
    if (part == HOMEPAGE_ELEMENT_FORUMS) {
      menuItems += forumsMenuItem;
    } else if (part == HOMEPAGE_ELEMENT_RECIPE) {
      menuItems += recipeMenuItem;
    } else if (part == HOMEPAGE_ELEMENT_GRAPH) {
      menuItems += graphMenuItems;
    } else if (part == HOMEPAGE_ELEMENT_GAME) {
      menuItems += gameMenuItems;
    }
  }

  $j('#' + menuPanelID).html(
      ' <ul data-role="listview" data-theme="d">' +
          '   <li data-icon="false"><a data-direction="reverse" ' + (pageName != 'homePage' ? ' href="#homePage"' : ' onclick="closePanel(\'' + menuPanelID + '\');"') + '>Home</a></li> ' +
          menuItems  +
          gameRuleItem +
          faqItem +
          '   <li data-icon="false"><a data-direction="reverse" href="#loginPage" onclick="logout()">Log uit</a></li>' +
//          '   <li data-icon="false"><a rel="external" href="grafiek/highcharts3-jquery.html"><br/><br/>Grafiek test</a></li>' +
          ' </ul>'
  );

  // TODO bind to each page, or just once to the window?
  // Enable swipe to show menu
  $j('#' + pageName).bind('swipeleft', function () {
    $j.mobile.silentScroll(0);
    $j('#' + menuPanelID).panel("open");
  });

  $j('#' + menuPanelID)
      .on("panelbeforeopen", function(event, ui) {
        openPanelID = menuPanelID;
      })
      .on("panelbeforeclose", function(event, ui) {
        openPanelID = null;
      });
}

function closePanel(menuPanelID) {
  $j('#' + menuPanelID).panel("close");
}

function writeForumListItem(forum, backHref) {
  return '' +
      '<li data-icon="false">' +
//      '<li data-icon="false" class="clickable" data-href="#forumIndexPage?forumID=' + forum.id + '&forumName='+forum.name+'&backHref='+backHref+'">' +
      (forum.hot_topic == null ? '' : getUserImg(forum.hot_topic.post.user)) +
      ' <a href="#forumIndexPage?forumID=' + forum.id + '&forumName='+forum.name+'&backHref='+backHref+'">' +
      '  <h3>' + forum.name + '</h3>' +
      (forum.hot_topic == null ? '' : '  <p>Laatste reactie: ' + getDateString(forum.hot_topic.post.updated_at) + ' door ' + getFullName(forum.hot_topic.post.user) + ' in ' + forum.hot_topic.title + '</p>') +
      '  </a>' +
      '</li>';
}

function parseDateYYYYMMDD(dateString, separator) {
  var parts = dateString.split(separator);
  return new Date(parts[0], parts[1] - 1, parts[2]);
}

function parseDateDDMMYYYY(dateString, separator) {
  var parts = dateString.split(separator);
  return new Date(parts[2], parts[1] - 1, parts[0]);
}

// formats a javascript date object to yyyy-mm-dd
function formatDate(date) {
  var curr_date = date.getDate();
  var curr_month = date.getMonth() + 1; //Months are zero based
  if (curr_month < 10) {
    // Add prepending zero
    curr_month = "0" + curr_month;
  }
  var curr_year = date.getFullYear();
  return curr_date + "-" + curr_month + "-" + curr_year;
}

function getLastEnteredWeight(callback) {
  doGet(
      getServiceURL("/measurements/weight"),
      false,
      function (data) {
        var lastEnteredWeight = defaultWeight;
        if ($j(data.result.measurements).length > 0) {
          var mostRecentMeasurement = $j(data.result.measurements.last());
          lastEnteredWeight = mostRecentMeasurement[0].value;
        }
        callback(Math.floor(lastEnteredWeight)-4);
      });
}

function getTodaysEnteredWeight(callback) {
  doGet(
      getServiceURL("/measurements/weight"),
      true,
      function (data) {
        var today = getToday();
        var todaysValue = null;
        $j(data.result.measurements).each(function (measurementIdx, measurement) {
          var measurementDate = parseDateYYYYMMDD(measurement.created_on, "/");
          if (measurementDate >= today) {
            todaysValue = measurement.value;
          }
        });
        callback(todaysValue);
      });
}

function navigateToHomePage() {
  $j.mobile.changePage($j("#homePage"));
}

function navigateToWeightGraphPage() {
  $j.mobile.changePage($j("#weightGraphPage"));
}

function loadMobileGraph() {
  // override this method of graphic.js, so we can call the server via our own secure method while leaving the graphic.js file more like the original
  AppController.prototype.loadJSON = function (jsonUrl) {
    var that = this;
    doGet(
        jsonUrl,
        false,
        function (data) {
          that.json = data;
          that.initApp();
        })
  };
  kickOffGraph();
}

function removeTags(html) {
	return html.replace(/(<([^>]+)>)/ig,"");
}

function localiseNumber(number) {
  return number.toString().replace('.', ',');
}

function age(dateString) {
  var birthday = getDate(dateString);
  return ~~((Date.now() - birthday) / (31557600000));
}

function getProfilePropertyText(property, result_json) {
	var text = null;
	switch(property) {
      case "sex" :
				if (result_json.sex!=null) text = (result_json.sex) ? 'Man'  + ' (' + age(result_json.date_of_birth) + ')' : 'Vrouw' + ' (' + age(result_json.date_of_birth) + ')';
				break;
			case "city" :
				if (result_json.address.city!=null && result_json.address.city.length>0) text = result_json.address.city;
				break;
			case "family_status" :
				if (result_json.family_status!=null && result_json.family_status.length>0) text = result_json.family_status;
				break;
			case "children" :
				if (result_json.children!=null) text = (result_json.children) ? 'Heeft kinderen' : 'Heeft geen kinderen';
				break;
			case "favorite_sport" :
				if (result_json.favorite_sport!=null && result_json.favorite_sport.length>0) text = "Sport: " + result_json.favorite_sport;
				break;
			case "occupation" :
				if (result_json.occupation!=null && result_json.occupation.length>0) text = result_json.occupation;
				break;
	}
	return text;
}
function getProfilePropertyImage(property, result_json) {
	var image = "blank_icon";
	switch(property) {
      case "sex" :
				if (result_json.sex!=null) image = (result_json.sex) ? "male" : "female";
				break;
			case "city" :
				image = "city";
				break;
			case "family_status" :
				image = "family_status";
				break;
			case "children" :
				image = "children";
				break;
			case "favorite_sport" :
				image = "sport";
				break;
			case "occupation" :
				image = "occupation";
				break;
	}
	return "img/"+image+".png";
}